{*************************************************************************}
{ GH Freebrary                                                            }
{                                                                         }
{ GHFCustomDataSetEvents.pas - TghCustomDataSetEvents class unit.         }
{                                                                         }
{ This software is distributed under the open source BSD license.         }
{                                                                         }
{ Copyright (c) 2002-2013, Al Gonzalez (rescatandoadelphi.blogspot.com)   }
{ All rights reserved.                                                    }
{                                                                         }
{ Redistribution and use in source and binary forms, with or without      }
{ modification, are permitted provided that the following conditions are  }
{ met:                                                                    }
{                                                                         }
{   - Redistributions of source code must retain the above copyright      }
{     notice, this list of conditions and the following disclaimer.       }
{   - Redistributions in binary form must reproduce the above copyright   }
{     notice, this list of conditions and the following disclaimer in the }
{     documentation and/or other materials provided with the              }
{     distribution.                                                       }
{   - Neither the name of Al Gonzalez nor the names of his contributors   }
{     may be used to endorse or promote products derived from this        }
{     software without specific prior written permission.                 }
{                                                                         }
{ THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS }
{ IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED   }
{ TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A         }
{ PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT      }
{ HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,  }
{ SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT        }
{ LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,   }
{ DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY   }
{ THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT     }
{ (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE   }
{ OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.    }
{*************************************************************************}

{ Please contribute to improve this library.  I am not a good programmer,
  but good programmers know what Delphi is worth.

  A.G. }

Unit GHFCustomDataSetEvents;  { Custom Data Set Events }

Interface

  Uses
    GHFDataSetEventHolder, Data.DB;

  Type
    { Custom Data Set Events class }
    TghCustomDataSetEvents = Class (TghDataSetEventHolder)
      Protected
        { Static methods }
        Function GetDataSet :TDataSet; Virtual; Abstract;
        Procedure SetDataSet (Const Value :TDataSet);

        { Virtual methods }
        Procedure InternalSetDataSet (Value :TDataSet); Virtual; Abstract;
      Public
        Destructor Destroy; Override;

        { Properties }
        Property DataSet :TDataSet Read GetDataSet Write SetDataSet;
    End;

Implementation

  Uses
    System.Classes;

  { TghCustomDataSetEvents }

  Destructor TghCustomDataSetEvents.Destroy;
  Begin
    DataSet := Nil;
    Inherited Destroy;
  End;

  { Protected static methods }

  { NOTE: This method assumes that the Value data set will not conflict
    with other TghCustomDataSetEvents objects.  Be aware that restoration
    of event handlers is made by setting the values obtained from the event
    properties of the data set when it was assigned. }
  Procedure TghCustomDataSetEvents.SetDataSet (Const Value :TDataSet);
  Begin
    If Value = DataSet Then
      Exit;

    If (DataSet <> Nil) And Not (csDesigning In ComponentState) Then
      // We restore the event handlers of the current data set
      With DataSet Do
      Begin
        AfterCancel := FAfterCancel;
        AfterClose := FAfterClose;
        AfterDelete := FAfterDelete;
        AfterEdit := FAfterEdit;
        AfterInsert := FAfterInsert;
        AfterOpen := FAfterOpen;
        AfterPost := FAfterPost;
        AfterScroll := FAfterScroll;
        AfterRefresh := FAfterRefresh;
        BeforeCancel := FBeforeCancel;
        BeforeClose := FBeforeClose;
        BeforeDelete := FBeforeDelete;
        BeforeEdit := FBeforeEdit;
        BeforeInsert := FBeforeInsert;
        BeforeOpen := FBeforeOpen;
        BeforePost := FBeforePost;
        BeforeRefresh := FBeforeRefresh;
        BeforeScroll := FBeforeScroll;
        OnCalcFields := FOnCalcFields;
        OnDeleteError := FOnDeleteError;
        OnEditError := FOnEditError;

        If Not IsUniDirectional Then
          OnFilterRecord := FOnFilterRecord;

        OnNewRecord := FOnNewRecord;
        OnPostError := FOnPostError;
      End;

    InternalSetDataSet (Value);

    If (DataSet <> Nil) And Not (csDesigning In ComponentState) Then
      // We set the event handlers of the new data set
      With DataSet Do
      Begin
        FAfterCancel := AfterCancel;
        AfterCancel := DoAfterCancel;
        FAfterClose := AfterClose;
        AfterClose := DoAfterClose;
        FAfterDelete := AfterDelete;
        AfterDelete := DoAfterDelete;
        FAfterEdit := AfterEdit;
        AfterEdit := DoAfterEdit;
        FAfterInsert := AfterInsert;
        AfterInsert := DoAfterInsert;
        FAfterOpen := AfterOpen;
        AfterOpen := DoAfterOpen;
        FAfterPost := AfterPost;
        AfterPost := DoAfterPost;
        FAfterScroll := AfterScroll;
        AfterScroll := DoAfterScroll;
        FAfterRefresh := AfterRefresh;
        AfterRefresh := DoAfterRefresh;
        FBeforeCancel := BeforeCancel;
        BeforeCancel := DoBeforeCancel;
        FBeforeClose := BeforeClose;
        BeforeClose := DoBeforeClose;
        FBeforeDelete := BeforeDelete;
        BeforeDelete := DoBeforeDelete;
        FBeforeEdit := BeforeEdit;
        BeforeEdit := DoBeforeEdit;
        FBeforeInsert := BeforeInsert;
        BeforeInsert := DoBeforeInsert;
        FBeforeOpen := BeforeOpen;
        BeforeOpen := DoBeforeOpen;
        FBeforePost := BeforePost;
        BeforePost := DoBeforePost;
        FBeforeRefresh := BeforeRefresh;
        BeforeRefresh := DoBeforeRefresh;
        FBeforeScroll := BeforeScroll;
        BeforeScroll := DoBeforeScroll;
        FOnCalcFields := OnCalcFields;
        OnCalcFields := DoOnCalcFields;
        FOnDeleteError := OnDeleteError;
        OnDeleteError := DoOnDeleteError;
        FOnEditError := OnEditError;
        OnEditError := DoOnEditError;

        If Not IsUniDirectional Then
        Begin
          FOnFilterRecord := OnFilterRecord;
          OnFilterRecord := DoOnFilterRecord;
        End;

        FOnNewRecord := OnNewRecord;
        OnNewRecord := DoOnNewRecord;
        FOnPostError := OnPostError;
        OnPostError := DoOnPostError;
      End;
  End;

End.
