{*************************************************************************}
{ GH Freebrary                                                            }
{                                                                         }
{ GHFRTL.pas - Unit of RTL related elements.                              }
{                                                                         }
{ This software is distributed under the open source BSD license.         }
{                                                                         }
{ Copyright (c) 2002-2013, Al Gonzalez (rescatandoadelphi.blogspot.com)   }
{ All rights reserved.                                                    }
{                                                                         }
{ Redistribution and use in source and binary forms, with or without      }
{ modification, are permitted provided that the following conditions are  }
{ met:                                                                    }
{                                                                         }
{   - Redistributions of source code must retain the above copyright      }
{     notice, this list of conditions and the following disclaimer.       }
{   - Redistributions in binary form must reproduce the above copyright   }
{     notice, this list of conditions and the following disclaimer in the }
{     documentation and/or other materials provided with the              }
{     distribution.                                                       }
{   - Neither the name of Al Gonzalez nor the names of his contributors   }
{     may be used to endorse or promote products derived from this        }
{     software without specific prior written permission.                 }
{                                                                         }
{ THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS }
{ IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED   }
{ TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A         }
{ PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT      }
{ HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,  }
{ SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT        }
{ LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,   }
{ DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY   }
{ THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT     }
{ (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE   }
{ OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.    }
{*************************************************************************}

{ Please contribute to improve this library.  I am not a good programmer,
  but good programmers know what Delphi is worth.

  A.G. }

Unit GHFRTL;  { RTL }

{$Warn Implicit_String_Cast Off}
{$Warn WideChar_Reduced Off}

Interface

  Uses
    System.TypInfo, System.Classes, System.SysUtils, System.Types,
    WinAPI.Windows, System.RTLConsts, WinAPI.Messages;

  Const
    { Digits }
    ghdgBin = ['0'..'1'];  // Binary
    ghdgDecimal = ['0'..'9'];  // Decimal
    ghdgHex = ghdgDecimal + ['A'..'F'];  // Hexadecimal
    ghdgOctal = ['0'..'7'];  // Octal
    ghdgRoman = ['I', 'V', 'X', 'L', 'C', 'D', 'M'];  // Roman

    { Formats }
    ghfmISODate = 'yyyy-mm-dd';
    ghfmISOTime = 'hh:nn:ss';
    // ISO Date and Time
    ghfmISODateTime = ghfmISODate + '"T"' + ghfmISOTime;
    // Standard Date and Time
    ghfmStdDateTime = ghfmISODate + ' ' + ghfmISOTime;

    { Messages }

    { Range from wm_User + $400 up to wm_User + $5FF (512 values reserved
      by GH Freebrary) }
    ghmsMin = wm_User + $400;  // Minimum
    ghmsCloseModal = ghmsMin;
    ghmsMax = ghmsMin + $1FF;  // Maximum

  Type
    { Pointer types }
    PClass = ^TClass;
    PObject = ^TObject;

    { Character Case type }
    TghChrCase = (ghcaDefault, ghcaUndefined = ghcaDefault, ghcaLower,
      ghcaUpper);

    { Enumeration Procedure type.  NOTE: Preferably use "Const" or "Var"
      for Variant arguments, so these will not be copied (TghEnumProc is a
      generic prototype). }
    TghEnumProc = Procedure (Item :Pointer; Const Params :Variant;
      Var AResult :Boolean);

    {*******************************************************************}
    { Midpoint Rounding type.                                           }
    {                                                                   }
    { Unbiased (bankers):      -2.5 = -2  -1.5 = -2  1.5 = 2  2.5 = 2  }
    { Toward zero:              -2.5 = -2  -1.5 = -1  1.5 = 1  2.5 = 2  }
    { Toward infinity:          -2.5 = -3  -1.5 = -2  1.5 = 2  2.5 = 3  }
    { Toward positive infinity: -2.5 = -2  -1.5 = -1  1.5 = 2  2.5 = 3  }
    { Toward negative infinity: -2.5 = -3  -1.5 = -2  1.5 = 1  2.5 = 2  }
    {*******************************************************************}
    TghMidpointRounding = (ghmrUnbiased, ghmrZero, ghmrInfinity,
      ghmrPositive, ghmrNegative);

    { Parameter Pointer Method Type }
    TghParamPtrMethod = Function (Param :Pointer) :Pointer Of Object;

    { PPropInfo Array type }
    TghPPropInfoArr = Array Of PPropInfo;

    { Pointer Method type }
    TghPtrMethod = Function :Pointer Of Object;

    { String Compare type }
    TghStrCompare = (ghscOrdinal, ghscOrdinalAnyCase, ghscLocale,
      ghscLocaleAnyCase, ghscSimple, ghscSimpleAnyCase);

    { Three Parameters Pointer Method Type }
    TghThreeParamsPtrMethod = Function (Param1, Param2, Param3 :Pointer)
      :Pointer Of Object;

    { Two Characters type }
    TghTwoChrs = Packed Record
      Case Byte Of
        0 : (Value1, Value2 :Char);
        1 : (Values :Array [0..1] Of Char);
        2 : (LowValue, HighValue :Char);
    End;

    { Two Dates type }
    TghTwoDates = Packed Record
      Case Byte Of
        0 : (Value1, Value2 :TDateTime);
        1 : (Values :Array [0..1] Of TDateTime);
        2 : (LowValue, HighValue :TDateTime);
    End;

    { Two Floats type }
    TghTwoFloats = Packed Record
      Case Byte Of
        0 : (Value1, Value2 :Double);
        1 : (Values :Array [0..1] Of Double);
        2 : (LowValue, HighValue :Double);
    End;

    { Two Integers type }
    TghTwoInts = Packed Record
      Case Byte Of
        0 : (Value1, Value2 :Integer);
        1 : (Values :Array [0..1] Of Integer);
        2 : (LowValue, HighValue :Integer);
        3 : (Int64Value :Int64);
    End;

    { Two Parameters Pointer Method Type }
    TghTwoParamsPtrMethod = Function (Param1, Param2 :Pointer) :Pointer Of
      Object;

    { Two Strings type }
    TghTwoStrs = Packed Record
      Value1 :String;
      Value2 :String;
    End;

    { Two Words type }
    TghTwoWords = Packed Record
      Case Byte Of
        0 : (Value1, Value2 :Word);
        1 : (Values :Array [0..1] Of Word);
        2 : (LowValue, HighValue :Word);
        3 : (IntValue :Integer);
    End;

    { TghTwoWords Array type }
    TghTwoWordsArr = Array Of TghTwoWords;

  Var
    { Dummy variables }
    GHDUCardinal :Cardinal;
    GHDUDateTime :TDateTime;
    GHDUExtended :Extended;
    GHDUInt :Integer;
    GHDUInt64 :Int64;
    GHDUPANSIChar :PANSIChar;
    GHDUPChar :PChar;
    GHDUPtr :Pointer;

    { Formats }
    GHFMSQLDate :String = ghfmISODate;
    GHFMSQLTime :String = ghfmISOTime;
    GHFMSQLDateTime :String = ghfmStdDateTime;

    { Midpoint Rounding }
    GHMidpointRounding :TghMidpointRounding;

  { Functions and procedures }

  { Assign }
  Procedure ghAssign (Var AVar :Variant; Const Index :Integer;
    Const Value :Variant);

  { Call }
  Function ghCall (Const Routine :TghEnumProc; Const Item :Pointer;
    Const Params :Variant; Const DefaultResult :Boolean = False) :Boolean;
    Overload;

  { Call }
  Function ghCall (Const Obj :TObject; Const MethodAddress :Pointer)
    :Pointer; Overload;

  { Call }
  Function ghCall (Const Obj :TObject; Const MethodAddress, Param :Pointer)
    :Pointer; Overload;

  { Call }
  Function ghCall (Const Obj :TObject;
    Const MethodAddress, Param1, Param2 :Pointer) :Pointer; Overload;

  { Call }
  Function ghCall (Const Obj :TObject; Const MethodAddress, Param1, Param2,
    Param3 :Pointer) :Pointer; Overload;

  { Case }
  Function ghCase (Const Value :String) :TghChrCase;

  { Cased }
  Function ghCased (Const Value :String; Const Upper :Boolean) :String;
    Overload;

  { Cased }
  Function ghCased (Const Value :String; Const ACase :TghChrCase) :String;
    Overload;

  { Cased }
  Function ghCased (Const Value, Sample :String) :String; Overload;

  { Character  }
  Function ghChr (Const Value :Char) :ANSIChar; Overload;

  { Character  }
  Function ghChr (Const Value :ANSIChar) :Char; Overload;

  { Character  }
  Function ghChr (Const Value :String; Const Index :Integer = 1) :Char;
    Overload;

  { Character  }
  Function ghChr (Const Value :ANSIString; Const Index :Integer = 1)
    :ANSIChar; Overload;

  { Character  }
  Function ghChr (Const Value :WideString; Const Index :Integer = 1)
    :WideChar; Overload;

  { Character, End }
  Function ghChrEnd (Const Value :String; Const Pos :Integer = 1) :Char;

  { Clone }
  Function ghClone (Const Obj :TObject; Const Cls :TClass;
    Const Owner :TComponent = Nil; Const ExcludedProps :String = '')
    :TObject; Overload;

  { Clone }
  Function ghClone (Const Obj :TObject; Const Owner :TComponent = Nil;
    Const ExcludedProps :String = '') :TObject; Overload;

  { Clone }
  Function ghClone (Const Obj :TObject; Const ExcludedProps :String)
    :TObject; Overload;

  { Comma }
  Function ghComma (Const Value1, Value2 :String;
    Const IncludeEmpties :Boolean = False) :String;

  { Comma Integers }
  Function ghCommaInts (Const Values :Array Of Integer) :String;

  { Commas }
  Function ghCommas (Const Values :Array Of String;
    Const IncludeEmpties :Boolean = False) :String; Overload;

  { Commas }
  Function ghCommas (Const List :TStrings;
    Const IncludeEmpties :Boolean = False) :String; Overload;

  { Comma SQL Values }
  Function ghCommaSQLValues (Const Values :Variant) :String;

  { Compare }
  Function ghCompare (Const Value1, Value2 :String;
    Const CompareType :TghStrCompare) :Integer;

  { Comparar Simple }
  Function ghCompareSimple (Value1, Value2 :String;
    Const AnyCase :Boolean = False) :Integer;

  { Component }
  Function ghComponent (Const Owner :TComponent;
    Const Cls :TComponentClass; Const Recurse :Boolean = False)
    :TComponent;

  { Composition }
  Function ghComposition (Const Value :Char; Const Typing :Boolean = False)
    :String;

  { Concatenate }
  Function ghConcat (Const Value1, Value2 :String;
    Const Separator :String = ' ') :String; Overload;

  { Concatenate }
  Function ghConcat (Const Value1, Value2, Separator :String;
    Const IncludeEmpties :Boolean) :String; Overload;

  { Concatenate }
  Function ghConcat (Value1, Value2 :String; Const Separator :Char;
    Const IncludeEmpties :Boolean; Const CheckAffixes :Boolean) :String;
    Overload;

  { Concatenate }
  Function ghConcat (Value1, Value2 :String; Const Separator :String;
    Const IncludeEmpties :Boolean; Const CheckAffixes :Boolean) :String;
    Overload;

  { Concatenate }
  Function ghConcat (Const Values :Array Of String;
    Const Separator :String = ' '; Const IncludeEmpties :Boolean = False;
    Const CheckAffixes :Boolean = False) :String; Overload;

  { Concatenate }
  Function ghConcat (Const List :TStrings; Const Separator :String = ' ';
    Const IncludeEmpties :Boolean = False;
    Const CheckAffixes :Boolean = False) :String; Overload;

  { Concatenate Integers }
  Function ghConcatInts (Const Values :Array Of Integer;
    Const Separator :String = ' ') :String; Overload;

  { Concatenate Integers }
  Function ghConcatInts (Const Values :Array Of Integer;
    Const MinLength :Integer; Const PadChr :Char = '0';
    Const Separator :String = ' ') :String; Overload;

  { Concatenate Integers }
  Function ghConcatInts (Const Values :Array Of Integer;
    Const Format :String; Const Separator :String) :String; Overload;

  { Concatenate Paths }
  Function ghConcatPaths (Const Path1, Path2 :String;
    Const APathDelim :Char = PathDelim) :String;

  { Copy }
  Function ghCopy (Const Value :String; Const StartIndex :Integer) :String;
    Overload;

  { Copy }
  Function ghCopy (Const Value :String;
    Const StartIndex, EndIndex :Integer) :String; Overload;

  { Copy Property }
  Function ghCopyProp (Const Source, Dest :TObject; Const Prop :PPropInfo)
    :Boolean;

  { Copy Properties }
  Procedure ghCopyProps (Const Source, Dest :TObject;
    Const Excludeds :String = '');

  { Count }
  Function ghCount (Const Value :String; Const Chr :Char;
    Const StartIndex :Integer = 1; EndIndex :Integer = MaxInt) :Integer;
    Overload;

  { Count }
  Function ghCount (Const Value :String; Const Chrs :TSysCharSet;
    Const StartIndex :Integer = 1; EndIndex :Integer = MaxInt) :Integer;
    Overload;

  { Count String }
  Function ghCountStr (Const Value, SubStr :String;
    Const CompareType :TghStrCompare = ghscLocaleAnyCase) :Integer;

  { Count To }
  Function ghCountTo (Const Value :String; Const Chr :Char;
    Const EndIndex :Integer) :Integer; Overload;

  { Count To }
  Function ghCountTo (Const Value :String; Const Chrs :TSysCharSet;
    Const EndIndex :Integer) :Integer; Overload;

  { Decimal }
  Function ghDecimal (Const Value :Extended; Const Precision :Integer = 18;
    Const Decimals :Integer = 9999) :TFloatRec;

  { Delete Left }
  Function ghDeleteLeft (Const Value :String; Const Chr :Char) :String;
    Overload;

  { Delete Left }
  Function ghDeleteLeft (Const Value :String; Const Chrs :TSysCharSet)
    :String; Overload;

  { Delete Right }
  Function ghDeleteRight (Const Value :String; Const Chrs :Integer = 1)
    :String; Overload;

  { Delete Right }
  Function ghDeleteRight (Const Value :String; Const Chr :Char) :String;
    Overload;

  { Delete Right }
  Function ghDeleteRight (Const Value :String; Const Chrs :TSysCharSet)
    :String; Overload;

  { Directory }
  Function ghDir (Const Path :String; Const UpLevels :Integer = 1;
    Const APathDelim :Char = PathDelim) :String;

  { Directory Path }
  Function ghDirPath (Const Path :String; Const UpLevels :Integer;
    Const SubPath :String; Const APathDelim :Char = PathDelim) :String;
    Overload;

  { Directory Path }
  Function ghDirPath (Const Path, SubPath :String;
    Const APathDelim :Char = PathDelim) :String; Overload;

  { Directory Path }
  Function ghDirPath (Const UpLevels :Integer; Const SubPath :String;
    Const APathDelim :Char = PathDelim) :String; Overload;

  { Directory Path }
  Function ghDirPath (Const SubPath :String;
    Const APathDelim :Char = PathDelim) :String; Overload;

  { Div Ceil }
  Function ghDivCeil (Const Dividend, Divisor :Integer) :Integer;

  { Div and Mod }
  Function ghDivMod (Const Dividend, Divisor :Integer) :TghTwoInts;
    Overload;

  { Div and Mod }
  Function ghDivMod (Const Dividend, Divisor :Extended;
    Const Decimals :Integer = 0) :TghTwoFloats; Overload;

  { Div and Mod Safe }
  Function ghDivModSafe (Const Dividend, Divisor :Integer) :TghTwoInts;

  { Div Safe }
  Function ghDivSafe (Const Dividend, Divisor :Integer) :Integer;

  { Enumerate Components }
  Function ghEnumComponents (Const Owner :TComponent;
    Const Cls :TComponentClass; Const Recurse :Boolean;
    Const Routine :TghEnumProc; Const Params :Variant) :TComponent;
    Overload;

  { Enumerate Components }
  Function ghEnumComponents (Const Owner :TComponent;
    Const Cls :TComponentClass; Const Routine :TghEnumProc;
    Const Params :Variant) :TComponent; Overload;

  { Enumerate Components }
  Function ghEnumComponents (Const Owner :TComponent;
    Const Recurse :Boolean; Const Routine :TghEnumProc;
    Const Params :Variant) :TComponent; Overload;

  { Enumerate Components }
  Function ghEnumComponents (Const Owner :TComponent;
    Const Routine :TghEnumProc; Const Params :Variant) :TComponent;
    Overload;

  { Enumerate Components }
  Function ghEnumComponents (Const Owner :TComponent;
    Const Cls :TComponentClass; Const Recurse :Boolean;
    Const Routine :TghEnumProc) :TComponent; Overload;

  { Enumerate Components }
  Function ghEnumComponents (Const Owner :TComponent;
    Const Cls :TComponentClass; Const Routine :TghEnumProc) :TComponent;
    Overload;

  { Enumerate Components }
  Function ghEnumComponents (Const Owner :TComponent;
    Const Recurse :Boolean; Const Routine :TghEnumProc) :TComponent;
    Overload;

  { Enumerate Components }
  Function ghEnumComponents (Const Owner :TComponent;
    Const Routine :TghEnumProc) :TComponent; Overload;

  { Enumerate Properties }
  Function ghEnumProps (Const Props :TghPPropInfoArr;
    Const Kinds :TTypeKinds; Const Routine :TghEnumProc;
    Const Params :Variant) :Integer; Overload;

  { Enumerate Properties }
  Function ghEnumProps (Const Obj :TObject; Const Kinds :TTypeKinds;
    Const Routine :TghEnumProc; Const Params :Variant) :Integer; Overload;

  { Enumerate Properties }
  Function ghEnumProps (Const Obj :TObject; Const Kinds :TTypeKinds;
    Const Routine :TghEnumProc = Nil) :Integer; Overload;

  { Enumerate Properties Object }
  Function ghEnumPropsObj (Const Obj :TObject; Const Routine :TghEnumProc;
    Const Params :Variant) :Integer; Overload;

  { Enumerate Properties Object }
  Function ghEnumPropsObj (Const Obj :TObject; Const Routine :TghEnumProc)
    :Integer; Overload;

  { Enumerate Properties Object }
  Function ghEnumPropsObj (Const Props :TghPPropInfoArr; Const Cls :TClass;
    Const Routine :TghEnumProc; Const Params :Variant) :Integer; Overload;

  { Enumerate Properties Object }
  Function ghEnumPropsObj (Const Obj :TObject; Const Cls :TClass;
    Const Routine :TghEnumProc; Const Params :Variant) :Integer; Overload;

  { Equals? }
  Function ghEquals (Const Value1, Value2 :String;
    Const CompareType :TghStrCompare = ghscLocaleAnyCase) :Boolean;
    Overload;

  { Equals? }
  Function ghEquals (Const Value1 :String; Const StartIndex :Integer;
    Const Value2 :String;
    Const CompareType :TghStrCompare = ghscLocaleAnyCase) :Boolean;
    Overload;

  { Equals? }
  Function ghEquals (Const Value1, Value2 :Variant) :Boolean; Overload;

  { Executable }
  Function ghExe :String;

  { Executable Directory }
  Function ghExeDir (Const UpLevels :Integer = 1;
    Const APathDelim :Char = PathDelim) :String;

  { Executable Name }
  Function ghExeName :String;

  { Format Optional }
  Function ghFormatOptional (Const AFormat, Param :String) :String;

  { Fraction }
  Function ghFrac (Const Value :Extended; Const Decimals :Integer)
    :Extended; Overload;

  { Fraction }
  Function ghFrac (Const Value :Extended; Const Decimal :Boolean;
    Const Decimals :Integer = MaxInt) :Extended; Overload;

  { Fraction Decimal }
  Function ghFracDecimal (Const Value :Extended;
    Decimals :Integer = MaxInt) :Extended;

  { Fraction Round }
  Function ghFracRound (Const Value :Extended; Const Epsilon :Extended = 0)
    :Integer;

  { Fraction Zero }
  Function ghFracZero (Const Value :Extended;
    Const Decimal :Boolean = False; Const Epsilon :Extended = 0;
    Const Decimals :Integer = MaxInt) :Extended; Overload;

  { Fraction Zero }
  Function ghFracZero (Const Value :Extended; Const Decimal :Boolean;
    Const Decimals :Integer) :Extended; Overload;

  { Fraction Zero }
  Function ghFracZero (Const Value :Extended; Const Epsilon :Extended;
    Const Decimals :Integer = MaxInt) :Extended; Overload;

  { Fraction Zero }
  Function ghFracZero (Const Value :Extended; Const Decimals :Integer)
    :Extended; Overload;

  { Has Bits? }
  Function ghHasBits (Const Value, Mask :Integer) :Boolean;

  { High }
  Function ghHigh (Const Value :Variant; Const Dim :Integer = 1) :Integer;

  { Increment }
  Function ghInc (Var Int :Integer; Const Inc :Integer = 1) :Integer;
    Overload;

  { Increment }
  Function ghInc (Const Value :String; Const Inc :Integer = 1) :String;
    Overload;

  { Increment }
  Function ghInc (Const Value :Variant; Const Inc :Integer = 1) :Variant;
    Overload;

  { Increment Positive }
  Function ghIncPositive (Const Value :Integer; Const Inc :Integer = 1)
    :Integer;

  { Increment Units }
  Function ghIncUnits (Const Groups, Units :Integer;
    Const MinUnit :Integer = 0; Const MaxUnit :Integer = 9;
    Const AInc :Integer = 1) :TghTwoInts; Overload;

  { Increment Units }
  Function ghIncUnits (Const Value :TghTwoInts;
    Const MinUnit :Integer = 0; Const MaxUnit :Integer = 9;
    Const AInc :Integer = 1) :TghTwoInts; Overload;

  { Index }
  Function ghIndex (Const Value :String; Const Chr :Char;
    Const StartIndex :Integer = 1) :Integer; Overload;

  { Index }
  Function ghIndex (Const Value :String; Const Chrs :TSysCharSet;
    Const StartIndex :Integer = 1) :Integer; Overload;

  { Index }
  Function ghIndex (Const List :TStrings; Const Str :String;
    Const StartIndex :Integer = 0;
    Const CompareType :TghStrCompare = ghscLocaleAnyCase) :Integer;
    Overload;

  { Index }
  Function ghIndex (Const List :TStrings; Const Str :String;
    Const CompareType :TghStrCompare) :Integer; Overload;

  { Index A Last }
  Function ghIndexALast (Const Value :String; Const Chr :Char;
    Occurrence :Integer; Const StartIndex :Integer = 1) :Integer;

  { Index Left }
  Function ghIndexLeft (Const Value :String; Const Chr :Char;
    Const StartIndex :Integer = 1) :Integer;

  { Index Maximum }
  Function ghIndexMax (Const Value, Count :Integer) :Integer;

  { Index Minimum }
  Function ghIndexMin (Const Value :Integer) :Integer;

  { Inherits From? }
  Function ghInheritsFrom (Cls :TClass; Const Name :String) :Boolean;

  { Integer }
  Function ghInt (Const Value :String) :Integer; Overload;

  { Integer }
  Function ghInt (Const Value :String; Const StartIndex :Integer) :Integer;
    Overload;

  { Integer }
  Function ghInt (Const Value :String; Const StartIndex, Chrs :Integer)
    :Integer; Overload;

  { Integer }
  Function ghInt (Const Value :Variant) :Integer; Overload;

  { Integer Even? }
  Function ghIntEven (Const Value :Extended) :Boolean;

  { Integer Strings }
  Function ghIntStrs (Const Values :Array Of Integer) :TStringDynArray;
    Overload;

  { Integer Strings }
  Function ghIntStrs (Const Values :Array Of Integer;
    Const MinLength :Integer; Const PadChr :Char = '0') :TStringDynArray;
    Overload;

  { Integer Strings }
  Function ghIntStrs (Const Values :Array Of Integer;
    Const AFormat :String) :TStringDynArray; Overload;

  { Is Characters? }
  Function ghIsChrs (Const Value :String; Const Chrs :TSysCharSet)
    :Boolean;

  { Is Diacritic? }
  Function ghIsDiacritic (Const Value :Char) :Boolean;

  { Is Hexadecimal? }
  Function ghIsHex (Const Value :Char) :Boolean; Overload;

  { Is Hexadecimal? }
  Function ghIsHex (Const Value :String) :Boolean; Overload;

  { Is Index? }
  Function ghIsIndex (Const Value, Count :Integer) :Boolean; Overload;

  { Is Index? }
  Function ghIsIndex (Const Value :Integer; Const Str :String) :Boolean;
    Overload;

  { Is Index? }
  Function ghIsIndex (Const Value :Integer; Const Str :ANSIString)
    :Boolean; Overload;

  { Is Index? }
  Function ghIsIndex (Const Value :Integer; Const Str :WideString)
    :Boolean; Overload;

  { Is Integer? }
  Function ghIsInt (Const Value :Extended; Const Epsilon :Extended = 0)
    :Boolean; Overload;

  { Is Integer? }
  Function ghIsInt (Const Value :String) :Boolean; Overload;

  { Is Lower? }
  Function ghIsLower (Const Value :String) :Boolean;

  { ISO Date }
  Function ghISODate (Const Value :TDateTime) :String; Overload;

  { ISO Date }
  Function ghISODate :String; Overload;

  { ISO Date and Time }
  Function ghISODateTime (Const Value :TDateTime) :String; Overload;

  { ISO Date and Time }
  Function ghISODateTime :String; Overload;

  { ISO Time }
  Function ghISOTime (Const Value :TDateTime) :String; Overload;

  { ISO Time }
  Function ghISOTime :String; Overload;

  { Is Pointer? }
  Function ghIsPtr (Const Value :String) :Boolean;

  { Is Upper? }
  Function ghIsUpper (Const Value :String) :Boolean;

  { Is Zero Or? }
  Function ghIsZeroOr (Const Value, Other :Extended;
    Const Epsilon :Extended = 0) :Boolean;

  { Last Error Message }
  Function ghLastErrorMsg :String;

  { Left }
  Function ghLeft (Const Value :String; Const Chr :Char;
    Const StartIndex :Integer = 1) :String; Overload;

  { Left }
  Function ghLeft (Const Value, SubStr :String;
    Const StartIndex :Integer = 1;
    Const CompareType :TghStrCompare = ghscLocaleAnyCase) :String;
    Overload;

  { Left }
  Function ghLeft (Const Value, SubStr :String;
    Const CompareType :TghStrCompare) :String; Overload;

  { Length }
  Function ghLength (Const Value :Variant; Const Dim :Integer = 1)
    :Integer;

  { Length of All }
  Function ghLengthAll (Const Value :Variant) :Integer;

  { Length of Dimension }
  Function ghLengthDim (Const Value :Variant; Const Dim :Integer) :Integer;

  { Low }
  Function ghLow (Const Value :Variant; Const Dim :Integer = 1) :Integer;

  { Maximum }
  Function ghMax (Const Value1, Value2 :TghTwoInts) :TghTwoInts;

  { Maximum Split Integer }
  Function ghMaxSplitInt (Const Value :String;
    Const Separator :String = ' '; Const IncludeEmpties :Boolean = False)
    :Integer;

  { Message Pending? }
  Function ghMsgPending (Const Window :THandle;
    Const MinID, MaxID :Integer) :Boolean; Overload;

  { Message Pending? }
  Function ghMsgPending (Const Window :THandle = 0; Const ID :Integer = 0)
    :Boolean; Overload;

  { Method Record }
  Function ghMethodRec (Const Obj, Address :Pointer) :TMethod;

  { Minimum }
  Function ghMin (Const Value1, Value2 :TghTwoInts) :TghTwoInts;

  { Minimum and Maximum }
  Function ghMinMax (Const Value1, Value2 :Integer) :TghTwoInts;

  { Minimum Zero }
  Function ghMinZero (Const Value :Integer) :Integer;

  { Module Path }
  Function ghModulePath (Const Module :THandle) :String; Overload;

  { Module Path }
  Function ghModulePath :String; Overload;

  { More than Null? }
  Function ghMoreNull (Const Value :Variant) :Boolean;

  { Peek Message }
  Function ghPeekMsg (Out Msg :TMsg; Const Window :THandle;
    Const MinID, MaxID :Integer; Const Remove :Boolean = False) :Boolean;
    Overload;

  { Peek Message }
  Function ghPeekMsg (Out Msg :TMsg; Const Window :THandle = 0;
    Const ID :Integer = 0; Const Remove :Boolean = False) :Boolean;
    Overload;

  { Percent }
  Function ghPercent (Const Value, Percentage :Extended) :Extended;
    Overload;

  { Percent }
  Function ghPercent (Const Value, Percentage :Extended;
    Const Decimals :Integer) :Extended; Overload;

  { Percent 100 }
  Function ghPercent100 (Const Value, Percentage :Extended) :Extended;
    Overload;

  { Percent 100 }
  Function ghPercent100 (Const Value, Percentage :Extended;
    Const Decimals :Integer) :Extended; Overload;

  { Percentage }
  Function ghPercentage (Const Value, Portion :Extended) :Extended;
    Overload;

  { Percentage }
  Function ghPercentage (Const Value, Portion :Extended;
    Const Decimals :Integer) :Extended; Overload;

  { Prepare Position }
  Function ghPreparePos (Var Str1, Str2 :String;
    Const CompareType :TghStrCompare = ghscLocaleAnyCase) :Boolean;

  { Position }
  Function ghPos (Value, SubStr :String; Const StartIndex :Integer = 1;
    Const CompareType :TghStrCompare = ghscLocaleAnyCase) :Integer;
    Overload;

  { Position }
  Function ghPos (Const Value, SubStr :String;
    Const CompareType :TghStrCompare) :Integer; Overload;

  { Position Left }
  Function ghPosLeft (Const Value, SubStr :String;
    Const StartIndex :Integer = 1;
    Const CompareType :TghStrCompare = ghscLocaleAnyCase) :Integer;

  { Position of String }
  Function ghPosStr (Const Values :Array Of String; Const Str :String;
    Const StartPos :Integer = 0;
    Const CompareType :TghStrCompare = ghscLocaleAnyCase) :Integer;
    Overload;

  { Position of String }
  Function ghPosStr (Const Values :Array Of String; Const Str :String;
    Const CompareType :TghStrCompare) :Integer; Overload;

  { Property Object }
  Function ghPropObj (Const Obj :TObject; Const Name :String;
    Const Cls :TClass = Nil) :TObject; Overload;

  { Property Object }
  Function ghPropObj (Const Obj :TObject; Const Cls :TClass = Nil)
    :TObject; Overload;

  { Properties }
  Function ghProps (Const Cls :TClass) :TghPPropInfoArr; Overload;

  { Properties }
  Function ghProps (Const Obj :TObject) :TghPPropInfoArr; Overload;

  { Property String }
  Function ghPropStr (Const Obj :TObject; Const Name :String) :String;

  { Property Value }
  Function ghPropValue (Const Obj :TObject; Const Name :String) :Variant;

  { Pointer }
  Function ghPtr (Const Value :String) :Pointer; Overload;

  { Pointer }
  Function ghPtr (Const Value :Variant) :Pointer; Overload;

  { Quoted ISO Date }
  Function ghQuotedISODate (Const Value :TDateTime) :String; Overload;

  { Quoted ISO Date }
  Function ghQuotedISODate :String; Overload;

  { Quoted ISO Date and Time }
  Function ghQuotedISODateTime (Const Value :TDateTime) :String; Overload;

  { Quoted ISO Date and Time }
  Function ghQuotedISODateTime :String; Overload;

  { Quoted ISO Time }
  Function ghQuotedISOTime (Const Value :TDateTime) :String; Overload;

  { Quoted ISO Time }
  Function ghQuotedISOTime :String; Overload;

  { Quoted SQL Date }
  Function ghQuotedSQLDate (Const Value :TDateTime) :String; Overload;

  { Quoted SQL Date }
  Function ghQuotedSQLDate :String; Overload;

  { Quoted SQL Date and Time }
  Function ghQuotedSQLDateTime (Const Value :TDateTime) :String; Overload;

  { Quoted SQL Date and Time }
  Function ghQuotedSQLDateTime :String; Overload;

  { Quoted SQL Time }
  Function ghQuotedSQLTime (Const Value :TDateTime) :String; Overload;

  { Quoted SQL Time }
  Function ghQuotedSQLTime :String; Overload;

  { Raise }
  Procedure ghRaise (Const Error :String; Const Callers :Integer = 1);
    Overload;

  { Raise }
  Procedure ghRaise (Const Error :String; Const Params :Array Of Const;
    Const Callers :Integer = 1); Overload;

  { Raise Index }
  Procedure ghRaiseIndex (Const Value :Integer;
    Const Callers :Integer = 1);

  { Raise Last Error }
  Procedure ghRaiseLastError (
    Const AFormat :String = '%s (GetLastError %d).';
    Const Callers :Integer = 1);

  { Reread Short String }
  Procedure ghRereadShortStr (Const Reader :TReader;
    Var ARightStr :String);

  { Return Address }
  Function ghReturnAddress (Callers :Integer = 1) :Pointer;

  { Right of Characters }
  Function ghRightChrs (Const Value :String; Const Chrs :TSysCharSet;
    Const MaxLength :Integer = MaxInt) :String;

  { Right of Digits }
  Function ghRightDigits (Const Value :String;
    Const MaxLength :Integer = MaxInt) :String;

  { Right Integer }
  Function ghRightInt (Const Value :String;
    Const MaxLength :Integer = MaxInt) :Integer;

  { Round }
  Function ghRound (Const Value :Extended; Const Decimals :Integer = 0;
    Const Epsilon :Extended = 0) :Extended;

  { Round Integer }
  Function ghRoundInt (Const Value :Extended; Const Epsilon :Extended = 0)
    :Int64;

  { Round to Multiple }
  Function ghRoundMultiple (Const Value, Divisor :Extended;
    Const Epsilon :Extended = 0) :Extended;

  { Set Character }
  Function ghSetChr (Var Str :String; Const Index :Integer;
    Const Value :Char) :Boolean;

  { Set Subcomponent }
  Procedure ghSetSubcomponent (Const Component :TComponent;
    Const Name :String);

  { Simple }
  Function ghSimple (Const Value :Char) :Char;

  { Simple String }
  Function ghSimpleStr (Const Value :String;
    Const DiacriticIndex :Integer) :String; Overload;

  { Simple String }
  Function ghSimpleStr (Const Value :String) :String; Overload;

  { Simple String Upper case }
  Function ghSimpleStrUpper (Const Value :String) :String;

  { Simplify }
  Function ghSimplify (Var Str :String; Const Index :Integer) :Boolean;

  { Simplify Safe }
  Function ghSimplifySafe (Var Str :String; Const Index :Integer) :Boolean;

  { Split }
  Function ghSplit (Const Value :String; Const Separator :String = ' ';
    Const IncludeEmpties :Boolean = False) :TStringDynArray;

  { Split Integers }
  Function ghSplitInts (Const Value :String; Const Separator :String = ' ';
    Const IncludeEmpties :Boolean = False) :TIntegerDynArray;

  { Split Last }
  Function ghSplitLast (Const Value :String; Const Separator :Char = ' ';
    Const IncludeEmpties :Boolean = False) :String;

  { SQL Date }
  Function ghSQLDate (Const Value :TDateTime) :String; Overload;

  { SQL Date }
  Function ghSQLDate :String; Overload;

  { SQL Date and Time }
  Function ghSQLDateTime (Const Value :TDateTime) :String; Overload;

  { SQL Date and Time }
  Function ghSQLDateTime :String; Overload;

  { SQL Time }
  Function ghSQLTime (Const Value :TDateTime) :String; Overload;

  { SQL Time }
  Function ghSQLTime :String; Overload;

  { SQL Value }
  Function ghSQLValue (Const Value :Variant) :String;

  { SQL Values }
  Function ghSQLValues (Const Values :Variant) :TStringDynArray;

  { Standard Date and Time }
  Function ghStdDateTime (Const Value :TDateTime) :String; Overload;

  { Standard Date and Time }
  Function ghStdDateTime :String; Overload;

  { String }
  Function ghStr (Const Value, MinLength :Integer;
    Const PadChr :Char = '0') :String; Overload;

  { String }
  Function ghStr (Const Value :Pointer) :String; Overload;

  { Strings }
  Function ghStrs (Const Values :Array Of String) :TStringDynArray;
    Overload;

  { Strings }
  Function ghStrs (Const List :TStrings) :TStringDynArray; Overload;

  { Swap Characters }
  Function ghSwapChrs (Const Value :TghTwoChrs) :TghTwoChrs;

  { Two Ints }
  Function ghTwoInts (Const Value1, Value2 :Integer) :TghTwoInts; Overload;

  { Two Ints }
  Function ghTwoInts (Const Values :Variant; Const StartIndex :Integer = 0)
    :TghTwoInts; Overload;

  { Type }
  Function ghType (Const Value :Char; Const Info :Integer = ct_CType1)
    :Word;

  { Unit Difference }
  Function ghUnitDiff (Const Groups1, Units1, Groups2, Units2 :Integer;
    Const MinUnit :Integer = 0; Const MaxUnit :Integer = 9) :Integer;
    Overload;

  { Unit Difference }
  Function ghUnitDiff (Const Value1, Value2 :TghTwoInts;
    Const MinUnit :Integer = 0; Const MaxUnit :Integer = 9) :Integer;
    Overload;

  { Units }
  Function ghUnits (Const Groups, Units :Integer;
    Const MinUnit :Integer = 0; Const MaxUnit :Integer = 9) :Integer;
    Overload;

  { Units }
  Function ghUnits (Const Value :TghTwoInts;
    Const MinUnit :Integer = 0; Const MaxUnit :Integer = 9) :Integer;
    Overload;

  { Valid? }
  Function ghValid (Const Value :String; Const Chrs :TSysCharSet) :Boolean;

  { Value }
  Function ghValue (Const AVar :Variant; Const Pos :Integer) :Variant;
    Overload;

  { Variant }
  Function ghVar (Const Value :String) :Variant; Overload;

  { Variant }
  Function ghVar (Const Value :Pointer) :Variant; Overload;

  { Variant }
  Function ghVar (Const Value :TVarRec) :Variant; Overload;

  { Variant }
  Function ghVar (Const Values :Array Of Const) :Variant; Overload;

  { Variant Array }
  Function ghVarArr (Const Size :Integer;
    Const AType :TVarType = varVariant;
    Const AllowLessThanTwo :Boolean = True) :Variant; Overload;

  { Variant Array }
  Function ghVarArr (Const Size :Integer;
    Const AllowLessThanTwo :Boolean) :Variant; Overload;

Implementation

  Uses
    System.Variants, System.StrUtils, System.Math;

  Var
    Exe :String;

  { Functions and procedures }

  Procedure ghAssign (Var AVar :Variant; Const Index :Integer;
    Const Value :Variant);
  Begin
    If VarIsArray (AVar) Then
      AVar [Index] := Value
    Else
      AVar := Value;
  End;

  Function ghCall (Const Routine :TghEnumProc; Const Item :Pointer;
    Const Params :Variant; Const DefaultResult :Boolean = False) :Boolean;
    Overload;
  Begin
    If Assigned (Routine) Then
    Begin
      Result := True;
      Routine (Item, Params, Result);
    End
    Else
      Result := DefaultResult;
  End;

  Function ghCall (Const Obj :TObject; Const MethodAddress :Pointer)
    :Pointer; Overload;
  Begin
    Result := TghPtrMethod (ghMethodRec (Obj, MethodAddress)) ();
  End;

  Function ghCall (Const Obj :TObject; Const MethodAddress, Param :Pointer)
    :Pointer; Overload;
  Begin
    Result := TghParamPtrMethod (ghMethodRec (Obj, MethodAddress)) (Param);
  End;

  Function ghCall (Const Obj :TObject;
    Const MethodAddress, Param1, Param2 :Pointer) :Pointer; Overload;
  Begin
    Result := TghTwoParamsPtrMethod (ghMethodRec (Obj, MethodAddress)) (
      Param1, Param2);
  End;

  Function ghCall (Const Obj :TObject; Const MethodAddress, Param1, Param2,
    Param3 :Pointer) :Pointer; Overload;
  Begin
    Result := TghThreeParamsPtrMethod (ghMethodRec (Obj, MethodAddress)) (
      Param1, Param2, Param3);
  End;

  Function ghCase (Const Value :String) :TghChrCase;
  Begin
    If ghIsUpper (Value) Then
      Result := ghcaUpper
    Else
      Result := TghChrCase (ghIsLower (Value));
  End;

  Function ghCased (Const Value :String; Const Upper :Boolean) :String;
    Overload;
  Begin
    If Upper Then
      Result := ANSIUpperCase (Value)
    Else
      Result := ANSILowerCase (Value);
  End;

  Function ghCased (Const Value :String; Const ACase :TghChrCase) :String;
    Overload;
  Begin
    If ACase = ghcaDefault Then
      Result := Value
    Else
      Result := ghCased (Value, ACase = ghcaUpper);
  End;

  Function ghCased (Const Value, Sample :String) :String; Overload;
  Begin
    Result := ghCased (Value, ghCase (Sample));
  End;

  Function ghChr (Const Value :Char) :ANSIChar; Overload;
  Begin
    Result := ANSIString (Value) [1];
  End;

  Function ghChr (Const Value :ANSIChar) :Char; Overload;
  Begin
    Result := String (Value) [1];
  End;

  Function ghChr (Const Value :String; Const Index :Integer = 1) :Char;
    Overload;
  Begin
    If ghIsIndex (Index, Value) Then
      Result := Value [Index]
    Else
      Result := #0;
  End;

  Function ghChr (Const Value :ANSIString; Const Index :Integer = 1)
    :ANSIChar; Overload;
  Begin
    If ghIsIndex (Index, Value) Then
      Result := Value [Index]
    Else
      Result := #0;
  End;

  Function ghChr (Const Value :WideString; Const Index :Integer = 1)
    :WideChar; Overload;
  Begin
    If ghIsIndex (Index, Value) Then
      Result := Value [Index]
    Else
      Result := #0;
  End;

  Function ghChrEnd (Const Value :String; Const Pos :Integer = 1) :Char;
  Begin
    Result := ghChr (Value, Length (Value) - Pred (Pos));
  End;

  Function ghClone (Const Obj :TObject; Const Cls :TClass;
    Const Owner :TComponent = Nil; Const ExcludedProps :String = '')
    :TObject; Overload;
  Begin
    If Obj Is TComponent Then
      Result := TComponentClass (Cls).Create (Owner)
    Else
      Result := Cls.Create;

    Try
      ghCopyProps (Obj, Result, ExcludedProps);
    Except
      Result.Free;
      Raise;
    End;
  End;

  Function ghClone (Const Obj :TObject; Const Owner :TComponent = Nil;
    Const ExcludedProps :String = '') :TObject; Overload;
  Begin
    Result := ghClone (Obj, Obj.ClassType, Owner, ExcludedProps);
  End;

  Function ghClone (Const Obj :TObject; Const ExcludedProps :String)
    :TObject; Overload;
  Begin
    Result := ghClone (Obj, Obj.ClassType, Nil, ExcludedProps);
  End;

  Function ghComma (Const Value1, Value2 :String;
    Const IncludeEmpties :Boolean = False) :String;
  Begin
    Result := ghConcat (Value1, Value2, ', ', IncludeEmpties);
  End;

  Function ghCommaInts (Const Values :Array Of Integer) :String;
  Begin
    Result := ghConcatInts (Values, ', ');
  End;

  Function ghCommas (Const Values :Array Of String;
    Const IncludeEmpties :Boolean = False) :String; Overload;
  Begin
    Result := ghConcat (Values, ', ', IncludeEmpties);
  End;

  Function ghCommas (Const List :TStrings;
    Const IncludeEmpties :Boolean = False) :String; Overload;
  Begin
    Result := ghCommas (ghStrs (List), IncludeEmpties);
  End;

  Function ghCommaSQLValues (Const Values :Variant) :String;
  Begin
    Result := ghCommas (ghSQLValues (Values), True);
  End;

  {$Warn No_RetVal Off}
  Function ghCompare (Const Value1, Value2 :String;
    Const CompareType :TghStrCompare) :Integer;
  Begin
    Case CompareType Of
      ghscOrdinal :
        Result := CompareStr (Value1, Value2);
      ghscOrdinalAnyCase :
        Result := CompareText (Value1, Value2);
      ghscLocale :
        Result := ANSICompareStr (Value1, Value2);
      ghscLocaleAnyCase :
        Result := ANSICompareText (Value1, Value2);
      ghscSimple :
        Result := ghCompareSimple (Value1, Value2);
      ghscSimpleAnyCase :
        Result := ghCompareSimple (Value1, Value2, True);
    End;
  End;
  {$Warn No_RetVal On}

  Function ghCompareSimple (Value1, Value2 :String;
    Const AnyCase :Boolean = False) :Integer;
  Begin
    If Length (Value1) = Length (Value2) Then
    Begin
      Value1 := ghSimpleStr (Value1);
      Value2 := ghSimpleStr (Value2);
    End;

    If AnyCase Then
      Result := ANSICompareText (Value1, Value2)
    Else
      Result := ANSICompareStr (Value1, Value2);
  End;

  Function ghComponent (Const Owner :TComponent;
    Const Cls :TComponentClass; Const Recurse :Boolean = False)
    :TComponent;
  Var
    I :Integer;
  Begin
    Result := Nil;

    For I := 0 To Owner.ComponentCount - 1 Do
    Begin
      If Owner.Components [I] Is Cls Then
        Result := Owner.Components [I]
      Else
        If Recurse Then
          Result := ghComponent (Owner.Components [I], Cls, True);

      If Result <> Nil Then
        Exit;
    End;
  End;

  Function ghComposition (Const Value :Char; Const Typing :Boolean = False)
    :String;
  Var
    AResult :TghTwoChrs;
    Chrs :Integer;
  Begin
    // NOTE: In Spanish, the letter  is not composite.
    If (Value In ['', '']) And (SysLocale.PriLangID = lang_Spanish) Then
      Result := Value
    Else
    Begin
      Chrs := FoldString (map_Composite, @Value, 1, @AResult, 2);

      If Typing Then  // For keyboard entries
        If Chrs = 2 Then
          // First the diacritic and then the letter
          AResult := ghSwapChrs (AResult)
        Else  // Chrs = 1
          If ghIsDiacritic (Value) Then  // It is only an accent mark
          Begin
            AResult.Value2 := ' ';  //  + space -> 
            Chrs := 2;
          End;

      SetString (Result, AResult.Values, Chrs);
    End;
  End;

  Function ghConcat (Const Value1, Value2 :String;
    Const Separator :String = ' ') :String; Overload;
  Begin
    If (Value1 <> '') And (Value2 <> '') Then
      Result := Value1 + Separator + Value2
    Else
      Result := Value1 + Value2;
  End;

  Function ghConcat (Const Value1, Value2, Separator :String;
    Const IncludeEmpties :Boolean) :String; Overload;
  Begin
    If IncludeEmpties Then
      Result := Value1 + Separator + Value2
    Else
      Result := ghConcat (Value1, Value2, Separator);
  End;

  Function ghConcat (Value1, Value2 :String; Const Separator :Char;
    Const IncludeEmpties :Boolean; Const CheckAffixes :Boolean) :String;
    Overload;
  Begin
    If CheckAffixes Then
    Begin
      If ghChrEnd (Value1) = Separator Then
        Value1 := ghDeleteRight (Value1);

      If ghChr (Value2) = Separator Then
        Value2 := ghCopy (Value2, 2)
    End;

    Result := ghConcat (Value1, Value2, Separator, IncludeEmpties);
  End;

  Function ghConcat (Value1, Value2 :String; Const Separator :String;
    Const IncludeEmpties :Boolean; Const CheckAffixes :Boolean) :String;
    Overload;
  Begin
    If CheckAffixes Then
    Begin
      If ANSIEndsStr (Separator, Value1) Then
        Value1 := ghDeleteRight (Value1, Length (Separator));

      If ANSIStartsStr (Separator, Value2) Then
        Value2 := ghCopy (Value2, Length (Separator) + 1);
    End;

    Result := ghConcat (Value1, Value2, Separator, IncludeEmpties);
  End;

  Function ghConcat (Const Values :Array Of String;
    Const Separator :String = ' '; Const IncludeEmpties :Boolean = False;
    Const CheckAffixes :Boolean = False) :String; Overload;
  Var
    I :Integer;
  Begin
    If Length (Values) > 0 Then
    Begin
      Result := Values [0];

      For I := 1 To High (Values) Do
        If (I > 1) And IncludeEmpties And CheckAffixes And
        ((Values [I - 1] = '') Or (Values [I - 1] = Separator)) Then
          { ['a', '', 'c'], ',', True, True -> 'a,' + ',c' = 'a,,c'
            ['a', ',', ',c'], ',', True, True -> 'a,' + ',c' = 'a,,c' }
          Result := Result + ghConcat ('', Values [I], Separator, True,
            True)
        Else
          Result := ghConcat (Result, Values [I], Separator,
            IncludeEmpties, CheckAffixes);
    End
    Else
      Result := '';
  End;

  Function ghConcat (Const List :TStrings; Const Separator :String = ' ';
    Const IncludeEmpties :Boolean = False;
    Const CheckAffixes :Boolean = False) :String; Overload;
  Begin
    Result := ghConcat (ghStrs (List), Separator, IncludeEmpties,
      CheckAffixes);
  End;

  Function ghConcatInts (Const Values :Array Of Integer;
    Const Separator :String = ' ') :String; Overload;
  Begin
    Result := ghConcat (ghIntStrs (Values), Separator);
  End;

  Function ghConcatInts (Const Values :Array Of Integer;
    Const MinLength :Integer; Const PadChr :Char = '0';
    Const Separator :String = ' ') :String; Overload;
  Begin
    Result := ghConcat (ghIntStrs (Values, MinLength, PadChr), Separator);
  End;

  Function ghConcatInts (Const Values :Array Of Integer;
    Const Format :String; Const Separator :String) :String; Overload;
  Begin
    Result := ghConcat (ghIntStrs (Values, Format), Separator);
  End;

  Function ghConcatPaths (Const Path1, Path2 :String;
    Const APathDelim :Char = PathDelim) :String;
  Begin
    Result := ghConcat (Path1, Path2, APathDelim, False, True);
  End;

  Function ghCopy (Const Value :String; Const StartIndex :Integer) :String;
    Overload;
  Begin
    Result := Copy (Value, StartIndex, MaxInt);
  End;

  Function ghCopy (Const Value :String;
    Const StartIndex, EndIndex :Integer) :String; Overload;
  Begin
    Result := Copy (Value, StartIndex,
      EndIndex - ghMinZero (StartIndex - 1));
  End;

  Function ghCopyProp (Const Source, Dest :TObject; Const Prop :PPropInfo)
    :Boolean;
  Begin
    If Prop.SetProc <> Nil  Then
    Begin
      Case Prop.PropType^.Kind Of
        { Object }
        tkClass : SetObjectProp (Dest, Prop, GetObjectProp (Source, Prop));
        { Event }
        tkMethod : SetMethodProp (Dest, Prop, GetMethodProp (Source, Prop))
        Else
          SetPropValue (Dest, Prop.Name, GetPropValue (Source, Prop.Name,
            False));
      End;

      Result := True;
    End
    Else
      Result := False;
  End;

  Procedure ghCopyProps (Const Source, Dest :TObject;
    Const Excludeds :String = '');
  Var
    ExcludedNames :TStringDynArray;
    Events :Boolean;
    I :Integer;
    Props :TghPPropInfoArr;
  Begin
    Props := ghProps (Dest);
    ExcludedNames := ghSplit (Excludeds, ';');

    // Substring '(events)' = not copy event properties
    Events := ghPosStr (ExcludedNames, '(events)') = -1;

    For I := 0 To High (Props) Do
      If (Events Or (Props [I].PropType^.Kind <> tkMethod)) And
      (ghPosStr (ExcludedNames, Props [I].Name) = -1) Then
        ghCopyProp (Source, Dest, Props [I]);
  End;

  Function ghCount (Const Value :String; Const Chr :Char;
    Const StartIndex :Integer = 1; EndIndex :Integer = MaxInt) :Integer;
    Overload;
  Var
    I :Integer;
  Begin
    If EndIndex = MaxInt Then
      EndIndex := Length (Value);

    Result := 0;

    For I := StartIndex To EndIndex Do
      If Value [I] = Chr Then
        Inc (Result);
  End;

  Function ghCount (Const Value :String; Const Chrs :TSysCharSet;
    Const StartIndex :Integer = 1; EndIndex :Integer = MaxInt) :Integer;
    Overload;
  Var
    I :Integer;
  Begin
    If EndIndex = MaxInt Then
      EndIndex := Length (Value);

    Result := 0;

    For I := StartIndex To EndIndex Do
      If Value [I] In Chrs Then
        Inc (Result);
  End;

  Function ghCountStr (Const Value, SubStr :String;
    Const CompareType :TghStrCompare = ghscLocaleAnyCase) :Integer;
  Var
    I :Integer;
  Begin
    Result := 0;
    I := 1;

    While ghIsIndex (I, Value) Do
    Begin
      I := ghPos (Value, SubStr, I, CompareType);

      If I > 0 Then
      Begin
        Inc (Result);
        Inc (I, Length (SubStr));
      End;
    End;
  End;

  Function ghCountTo (Const Value :String; Const Chr :Char;
    Const EndIndex :Integer) :Integer; Overload;
  Begin
    Result := ghCount (Value, Chr, 1, EndIndex);
  End;

  Function ghCountTo (Const Value :String; Const Chrs :TSysCharSet;
    Const EndIndex :Integer) :Integer; Overload;
  Begin
    Result := ghCount (Value, Chrs, 1, EndIndex);
  End;

  Function ghDecimal (Const Value :Extended; Const Precision :Integer = 18;
    Const Decimals :Integer = 9999) :TFloatRec;
  Begin
    FloatToDecimal (Result, Value, fvExtended, Precision, Decimals);
  End;

  Function ghDeleteLeft (Const Value :String; Const Chr :Char) :String;
    Overload;
  Var
    I :Integer;
  Begin
    I := 1;

    While (I <= Length (Value)) And (Value [I] = Chr) Do
      Inc (I);

    Result := ghCopy (Value, I);
  End;

  Function ghDeleteLeft (Const Value :String; Const Chrs :TSysCharSet)
    :String; Overload;
  Var
    I :Integer;
  Begin
    I := 1;

    While (I <= Length (Value)) And (Value [I] In Chrs) Do
      Inc (I);

    Result := ghCopy (Value, I);
  End;

  Function ghDeleteRight (Const Value :String; Const Chrs :Integer = 1)
    :String; Overload;
  Begin
    Result := LeftStr (Value, Length (Value) - Chrs);
  End;

  Function ghDeleteRight (Const Value :String; Const Chr :Char) :String;
    Overload;
  Var
    I :Integer;
  Begin
    I := Length (Value);

    While (I > 0) And (Value [I] = Chr) Do
      Dec (I);

    Result := LeftStr (Value, I);
  End;

  Function ghDeleteRight (Const Value :String; Const Chrs :TSysCharSet)
    :String; Overload;
  Var
    I :Integer;
  Begin
    I := Length (Value);

    While (I > 0) And (Value [I] In Chrs) Do
      Dec (I);

    Result := LeftStr (Value, I);
  End;

  Function ghDir (Const Path :String; Const UpLevels :Integer = 1;
    Const APathDelim :Char = PathDelim) :String;
  Var
    Index :Integer;
  Begin
    Index := ghIndexALast (Path, APathDelim, UpLevels + Byte (ghChrEnd (
      Path) = APathDelim), Length (ExtractFileDrive (Path)) + 1);

    If Index > 0 Then
      Result := LeftStr (Path,
        Index - Byte ((Index > 1) And (Path [Index - 1] <> DriveDelim)))
    Else
      Result := '';
  End;

  Function ghDirPath (Const Path :String; Const UpLevels :Integer;
    Const SubPath :String; Const APathDelim :Char = PathDelim) :String;
    Overload;
  Begin
    Result := ghConcatPaths (ghDir (Path, UpLevels, APathDelim), SubPath,
      APathDelim);
  End;

  Function ghDirPath (Const Path, SubPath :String;
    Const APathDelim :Char = PathDelim) :String; Overload;
  Begin
    Result := ghDirPath (Path, 1, SubPath, APathDelim);
  End;

  Function ghDirPath (Const UpLevels :Integer; Const SubPath :String;
    Const APathDelim :Char = PathDelim) :String; Overload;
  Begin
    Result := ghDirPath (ghExe, UpLevels, SubPath, APathDelim);
  End;

  Function ghDirPath (Const SubPath :String;
    Const APathDelim :Char = PathDelim) :String; Overload;
  Begin
    Result := ghDirPath (1, SubPath, APathDelim);
  End;

  Function ghDivCeil (Const Dividend, Divisor :Integer) :Integer;
  Begin
    Result := (Dividend + (Divisor - 1)) Div Divisor;
  End;

  Function ghDivMod (Const Dividend, Divisor :Integer) :TghTwoInts;
    Overload;
  Begin
    Result.HighValue := Dividend Div Divisor;
    Result.LowValue := Dividend Mod Divisor;
  End;

  Function ghDivMod (Const Dividend, Divisor :Extended;
    Const Decimals :Integer = 0) :TghTwoFloats; Overload;
  Begin
    { ghDivMod (10.24, 3, 2) -> High = 3.41, Low = 0.01
      (3.41 * 3) + 0.01 = 10.24 }

    // 10.24 / 3 = 3.41333... -> 3.41
    Result.HighValue := ghRound (Dividend / Divisor, Decimals);

    // 10.24 - (3.41 * 3) = 10.24 - 10.23 = 0.01
    Result.LowValue := Dividend - (Result.HighValue * Divisor);
  End;

  Function ghDivModSafe (Const Dividend, Divisor :Integer) :TghTwoInts;
  Begin
    If Divisor = 0 Then
      Result.Int64Value := 0
    Else
      Result := ghDivMod (Dividend, Divisor);
  End;

  Function ghDivSafe (Const Dividend, Divisor :Integer) :Integer;
  Begin
    If Divisor = 0 Then
      Result := 0
    Else
      Result := Dividend Div Divisor;
  End;

  Function ghEnumComponents (Const Owner :TComponent;
    Const Cls :TComponentClass; Const Recurse :Boolean;
    Const Routine :TghEnumProc; Const Params :Variant) :TComponent;
    Overload;
  Var
    I :Integer;
  Begin
    { NOTE: Use the TghObjectList's enumeration instead of this function
      if Routine can alter the Components/ComponentCount properties. }

    For I := 0 To Owner.ComponentCount - 1 Do
    Begin
      Result := Owner.Components [I];

      If ((Cls <> Nil) And Not (Result Is Cls)) Or
      ghCall (Routine, Result, Params) Then
        If Recurse Then
          Result := ghEnumComponents (Owner.Components [I], Cls, True,
            Routine, Params)
        Else
          Result := Nil;

      If Result <> Nil Then
        Exit;
    End;

    Result := Nil;
  End;

  Function ghEnumComponents (Const Owner :TComponent;
    Const Cls :TComponentClass; Const Routine :TghEnumProc;
    Const Params :Variant) :TComponent; Overload;
  Begin
    Result := ghEnumComponents (Owner, Cls, False, Routine, Params);
  End;

  Function ghEnumComponents (Const Owner :TComponent;
    Const Recurse :Boolean; Const Routine :TghEnumProc;
    Const Params :Variant) :TComponent; Overload;
  Begin
    Result := ghEnumComponents (Owner, TComponentClass (Nil), Recurse,
      Routine, Params);
  End;

  Function ghEnumComponents (Const Owner :TComponent;
    Const Routine :TghEnumProc; Const Params :Variant) :TComponent;
    Overload;
  Begin
    Result := ghEnumComponents (Owner, TComponentClass (Nil), False,
      Routine, Params);
  End;

  Function ghEnumComponents (Const Owner :TComponent;
    Const Cls :TComponentClass; Const Recurse :Boolean;
    Const Routine :TghEnumProc) :TComponent; Overload;
  Begin
    Result := ghEnumComponents (Owner, Cls, Recurse, Routine, Unassigned);
  End;

  Function ghEnumComponents (Const Owner :TComponent;
    Const Cls :TComponentClass; Const Routine :TghEnumProc) :TComponent;
    Overload;
  Begin
    Result := ghEnumComponents (Owner, Cls, False, Routine, Unassigned);
  End;

  Function ghEnumComponents (Const Owner :TComponent;
    Const Recurse :Boolean; Const Routine :TghEnumProc) :TComponent;
    Overload;
  Begin
    Result := ghEnumComponents (Owner, TComponentClass (Nil), Recurse,
      Routine, Unassigned);
  End;

  Function ghEnumComponents (Const Owner :TComponent;
    Const Routine :TghEnumProc) :TComponent; Overload;
  Begin
    Result := ghEnumComponents (Owner, TComponentClass (Nil), False,
      Routine, Unassigned);
  End;

  Function ghEnumProps (Const Props :TghPPropInfoArr;
    Const Kinds :TTypeKinds; Const Routine :TghEnumProc;
    Const Params :Variant) :Integer; Overload;
  Begin
    For Result := 0 To High (Props) Do
      If (Props [Result].PropType^.Kind In Kinds) And Not ghCall (Routine,
      Props [Result], Params) Then
        Exit;

    Result := -1;
  End;

  Function ghEnumProps (Const Obj :TObject; Const Kinds :TTypeKinds;
    Const Routine :TghEnumProc; Const Params :Variant) :Integer; Overload;
  Begin
    Result := ghEnumProps (ghProps (Obj), Kinds, Routine, Params);
  End;

  Function ghEnumProps (Const Obj :TObject; Const Kinds :TTypeKinds;
    Const Routine :TghEnumProc = Nil) :Integer; Overload;
  Begin
    Result := ghEnumProps (Obj, Kinds, Routine, Null);
  End;

  Function ghEnumPropsObj (Const Obj :TObject; Const Routine :TghEnumProc;
    Const Params :Variant) :Integer; Overload;
  Begin
    Result := ghEnumProps (Obj, [tkClass], Routine, Params);
  End;

  Function ghEnumPropsObj (Const Obj :TObject; Const Routine :TghEnumProc)
    :Integer; Overload;
  Begin
    Result := ghEnumPropsObj (Obj, Routine, Null);
  End;

  Function ghEnumPropsObj (Const Props :TghPPropInfoArr; Const Cls :TClass;
    Const Routine :TghEnumProc; Const Params :Variant) :Integer; Overload;
  Begin
    For Result := 0 To High (Props) Do
      If (Props [Result].PropType^.Kind = tkClass) And ((Cls = Nil) Or
      GetTypeData (Props [Result].PropType^).ClassType.InheritsFrom (
      Cls)) And Not ghCall (Routine, Props [Result], Params) Then
        Exit;

    Result := -1;
  End;

  Function ghEnumPropsObj (Const Obj :TObject; Const Cls :TClass;
    Const Routine :TghEnumProc; Const Params :Variant) :Integer; Overload;
  Begin
    Result := ghEnumPropsObj (ghProps (Obj), Cls, Routine, Params);
  End;

  Function ghEquals (Const Value1, Value2 :String;
    Const CompareType :TghStrCompare = ghscLocaleAnyCase) :Boolean;
    Overload;
  Begin
    Result := ghCompare (Value1, Value2, CompareType) = 0;
  End;

  Function ghEquals (Const Value1 :String; Const StartIndex :Integer;
    Const Value2 :String;
    Const CompareType :TghStrCompare = ghscLocaleAnyCase) :Boolean;
    Overload;
  Begin
    Result := ghEquals (ghCopy (Value1, StartIndex), Value2, CompareType);
  End;

  Function ghEquals (Const Value1, Value2 :Variant) :Boolean; Overload;
  Begin
    Result := (VarType (Value1) = VarType (Value2)) And (Value1 = Value2);
  End;

  Function ghExe :String;
  Begin
    If Exe = '' Then
      Exe := ParamStr (0);

    Result := Exe;
  End;

  Function ghExeDir (Const UpLevels :Integer = 1;
    Const APathDelim :Char = PathDelim) :String;
  Begin
    Result := ghDir (ghExe, UpLevels, APathDelim);
  End;

  Function ghExeName :String;
  Begin
    Result := ExtractFileName (ghExe);
  End;

  Function ghFormatOptional (Const AFormat, Param :String) :String;
  Begin
    Result := Format (AFormat, [ghIncPositive (Length (Param)), Param]);
  End;

  Function ghFrac (Const Value :Extended; Const Decimals :Integer)
    :Extended; Overload;
  Begin
    Result := Frac (Value);

    If Decimals < MaxInt Then
      Result := ghRound (Result, Decimals);
  End;

  Function ghFrac (Const Value :Extended; Const Decimal :Boolean;
    Const Decimals :Integer = MaxInt) :Extended; Overload;
  Begin
    If Decimal Then
      Result := ghFracDecimal (Value, Decimals)
    Else
      Result := ghFrac (Value, Decimals);
  End;

  Function ghFracDecimal (Const Value :Extended;
    Decimals :Integer = MaxInt) :Extended;
  Var
    Buffer :PANSIChar Absolute Result;
    Desc :TFloatRec;  // Decimal descriptor
  Begin
    { Frac (-123.0045) -> -0.0044999...
      ghFracDecimal (-123.0045) -> -0.0045 }

    Result := Frac (Value);

    If Result = 0 Then
      Exit;

    { If Value is not a fraction but has fractional part, then we try to
      get it based on its decimal representation }
    If Abs (Value) > 1 Then
    Begin
      Desc := ghDecimal (Value);

      { When Frac (Value) <> 0 and Exponent < 18, StrLen (Digits) is
        greater than Exponent.  StrLen (Digits) - Exponent = "represented"
        decimals in Value.  If Exponent >= 18 then Value has no
        "represented" decimals. }
      If Desc.Exponent < 18 Then
      Begin
        If (Decimals < MaxInt) And (Integer (StrLen (Desc.Digits)) -
        Desc.Exponent <= Decimals) Then
          Decimals := MaxInt;

        { -123.0045, Digits = ['1', '2', '3', '0', '0', '4', '5', #0...] ->
          ['1', '-', '.', '0', '0', '4', '5', #0...], Buffer = '-.0045'.
          NOTE: This operation can overwrite the Desc's Exponent and
          Negative fields. ---------------------------------------------- }

        Buffer := PANSIChar (@Desc.Digits) + (Desc.Exponent - 2);

        If Desc.Negative Then
          Buffer [0] := '-'
        Else
          Inc (Buffer);

        Buffer [Byte (Buffer [0] = '-')] :=
          ghChr (FormatSettings.DecimalSeparator);
        //-----------------------------------------------------------------

        TextToFloat (Buffer, Result, fvExtended);  // '-.0045' -> -0.0045
      End;
    End;

    If Decimals < MaxInt Then
      Result := ghRound (Result, Decimals);
  End;

  Function ghFracRound (Const Value :Extended; Const Epsilon :Extended = 0)
    :Integer;
  Begin
    Result := CompareValue (Abs (ghFracDecimal (Value)), 0.5, Epsilon);

    If Result = LessThanValue Then  // -1 -> 0, GreaterThanValue = 1
      Result := 0
    Else
      If Result = EqualsValue Then  // Fraction = 0.5 +/- Epsilon
        Case GHMidpointRounding Of
          ghmrUnbiased : Result := Byte (Not ghIntEven (Value));
          ghmrPositive : Result := Byte (Value > 0);
          ghmrNegative : Result := Byte (Value < 0);
          Else
            Result := Byte (GHMidpointRounding = ghmrInfinity);
        End;

    If (Result = 1) And (Value < 0) Then
      Result := -1;
  End;

  Function ghFracZero (Const Value :Extended;
    Const Decimal :Boolean = False; Const Epsilon :Extended = 0;
    Const Decimals :Integer = MaxInt) :Extended; Overload;
  Begin
    Result := ghFrac (Value, Decimal);

    If Result <> 0 Then
      If ghIsZeroOr (Abs (Result), 1, Epsilon) Then
        Result := 0  // 0.000... or 0.999... -> 0
      Else
        If Decimals < MaxInt Then
          // NOTE: We do not pass Epsilon here.
          Result := ghRound (Result, Decimals);
  End;

  Function ghFracZero (Const Value :Extended; Const Decimal :Boolean;
    Const Decimals :Integer) :Extended; Overload;
  Begin
    Result := ghFracZero (Value, Decimal, 0, Decimals);
  End;

  Function ghFracZero (Const Value :Extended; Const Epsilon :Extended;
    Const Decimals :Integer = MaxInt) :Extended; Overload;
  Begin
    Result := ghFracZero (Value, False, Epsilon, Decimals);
  End;

  Function ghFracZero (Const Value :Extended; Const Decimals :Integer)
    :Extended; Overload;
  Begin
    Result := ghFracZero (Value, False, 0, Decimals);
  End;

  Function ghHasBits (Const Value, Mask :Integer) :Boolean;
  Begin
    Result := (Mask <> 0) And ((Value And Mask) = Mask);
  End;

  Function ghHigh (Const Value :Variant; Const Dim :Integer = 1) :Integer;
  Begin
    If VarIsArray (Value) Then
      Result := VarArrayHighBound (Value, Dim)
    Else
      Result := 0;
  End;

  Function ghInc (Var Int :Integer; Const Inc :Integer = 1) :Integer;
    Overload;
  Begin
    Int := Int + Inc;
    Result := Int;
  End;

  Function ghInc (Const Value :String; Const Inc :Integer = 1) :String;
    Overload;
  Begin
    Result := ghRightDigits (Value);

    If Result <> '' Then
      Result := ghDeleteRight (Value, Length (Result)) +
        ghStr (StrToInt (Result) + Inc, Length (Result))
    Else
      Result := Value + IntToStr (Inc);
  End;

  Function ghInc (Const Value :Variant; Const Inc :Integer = 1) :Variant;
    Overload;
  Begin
    If VarIsNull (Value) Then
      Result := Inc
    Else
      If VarIsStr (Value) Then
        Result := ghInc (String (Value), Inc)
      Else
        Result := Value + Inc;
  End;

  Function ghIncPositive (Const Value :Integer; Const Inc :Integer = 1)
    :Integer;
  Begin
    If Value > 0 Then
      Result := Value + Inc
    Else
      Result := Value;
  End;

  Function ghIncUnits (Const Groups, Units :Integer;
    Const MinUnit :Integer = 0; Const MaxUnit :Integer = 9;
    Const AInc :Integer = 1) :TghTwoInts; Overload;
  Begin
    Result := ghDivMod (ghUnits (Groups, Units, MinUnit, MaxUnit) + AInc,
      MaxUnit - Pred (MinUnit));
    Inc (Result.LowValue, MinUnit);
  End;

  Function ghIncUnits (Const Value :TghTwoInts;
    Const MinUnit :Integer = 0; Const MaxUnit :Integer = 9;
    Const AInc :Integer = 1) :TghTwoInts; Overload;
  Begin
    Result := ghIncUnits (Value.HighValue, Value.LowValue, MinUnit,
      MaxUnit, AInc);
  End;

  Function ghIndex (Const Value :String; Const Chr :Char;
    Const StartIndex :Integer = 1) :Integer; Overload;
  Begin
    For Result := StartIndex To Length (Value) Do
      If Value [Result] = Chr Then
        Exit;

    Result := 0;
  End;

  Function ghIndex (Const Value :String; Const Chrs :TSysCharSet;
    Const StartIndex :Integer = 1) :Integer; Overload;
  Begin
    For Result := StartIndex To Length (Value) Do
      If Value [Result] In Chrs Then
        Exit;

    Result := 0;
  End;

  Function ghIndex (Const List :TStrings; Const Str :String;
    Const StartIndex :Integer = 0;
    Const CompareType :TghStrCompare = ghscLocaleAnyCase) :Integer;
    Overload;
  Begin
    For Result := StartIndex To List.Count - 1 Do
      If ghEquals (List [Result], Str, CompareType) Then
        Exit;

    Result := -1;
  End;

  Function ghIndex (Const List :TStrings; Const Str :String;
    Const CompareType :TghStrCompare) :Integer; Overload;
  Begin
    Result := ghIndex (List, Str, 0, CompareType);
  End;

  Function ghIndexALast (Const Value :String; Const Chr :Char;
    Occurrence :Integer; Const StartIndex :Integer = 1) :Integer;
  Begin
    For Result := Length (Value) DownTo StartIndex Do
      If (Value [Result] = Chr) And (ghInc (Occurrence, -1) = 0) Then
        Exit;

    Result := 0;
  End;

  Function ghIndexLeft (Const Value :String; Const Chr :Char;
    Const StartIndex :Integer = 1) :Integer;
  Begin
    Result := ghIndex (Value, Chr, StartIndex) - 1;

    If Result = -1 Then
      Result := Length (Value);
  End;

  Function ghIndexMax (Const Value, Count :Integer) :Integer;
  Begin
    Result := Min (Value, Count - 1);
  End;

  Function ghIndexMin (Const Value :Integer) :Integer;
  Begin
    Result := Max (Value, 0);
  End;

  Function ghInheritsFrom (Cls :TClass; Const Name :String) :Boolean;
  Begin
    While (Cls <> Nil) And Not Cls.ClassNameIs (Name) Do
      Cls := Cls.ClassParent;

    Result := Cls <> Nil;
  End;

  Function ghInt (Const Value :String) :Integer; Overload;
  Begin
    Result := StrToIntDef (Value, 0);
  End;

  Function ghInt (Const Value :String; Const StartIndex :Integer) :Integer;
    Overload;
  Begin
    Result := ghInt (ghCopy (Value, StartIndex));
  End;

  Function ghInt (Const Value :String; Const StartIndex, Chrs :Integer)
    :Integer; Overload;
  Begin
    Result := ghInt (Copy (Value, StartIndex, Chrs));
  End;

  Function ghInt (Const Value :Variant) :Integer; Overload;
  Begin
    If VarIsNull (Value) Then
      Result := 0
    Else
      If VarIsStr (Value) Then
        Result := ghInt (String (Value))
      Else
        Result := Trunc (Value);
  End;

  Function ghIntEven (Const Value :Extended) :Boolean;
  Begin
    Result := Frac (Int (Value) / 2) = 0;
  End;

  Function ghIntStrs (Const Values :Array Of Integer) :TStringDynArray;
    Overload;
  Var
    I :Integer;
  Begin
    SetLength (Result, Length (Values));

    For I := 0 To High (Values) Do
      Result [I] := IntToStr (Values [I]);
  End;

  Function ghIntStrs (Const Values :Array Of Integer;
    Const MinLength :Integer; Const PadChr :Char = '0') :TStringDynArray;
    Overload;
  Var
    I :Integer;
  Begin
    SetLength (Result, Length (Values));

    For I := 0 To High (Values) Do
      Result [I] := ghStr (Values [I], MinLength, PadChr);
  End;

  Function ghIntStrs (Const Values :Array Of Integer;
    Const AFormat :String) :TStringDynArray; Overload;
  Var
    I :Integer;
  Begin
    SetLength (Result, Length (Values));

    For I := 0 To High (Values) Do
      Result [I] := Format (AFormat, [Values [I]]);
  End;

  Function ghIsChrs (Const Value :String; Const Chrs :TSysCharSet)
    :Boolean;
  Begin
    Result := (Value <> '') And ghValid (Value, Chrs);
  End;

  Function ghIsDiacritic (Const Value :Char) :Boolean;
  Begin
    Result := ghHasBits (ghType (Value, ct_CType3),
      c3_Diacritic Or c3_Symbol);
  End;

  Function ghIsHex (Const Value :Char) :Boolean; Overload;
  Begin
    Result := UpCase (Value) In ghdgHex;
  End;

  Function ghIsHex (Const Value :String) :Boolean; Overload;
  Begin
    Result := ghIsChrs (UpperCase (Value), ghdgHex);
  End;

  Function ghIsIndex (Const Value, Count :Integer) :Boolean; Overload;
  Begin
    Result := (Value >= 0) And (Value < Count);
  End;

  Function ghIsIndex (Const Value :Integer; Const Str :String) :Boolean;
    Overload;
  Begin
    Result := (Value > 0) And (Value <= Length (Str));
  End;

  Function ghIsIndex (Const Value :Integer; Const Str :ANSIString)
    :Boolean; Overload;
  Begin
    Result := (Value > 0) And (Value <= Length (Str));
  End;

  Function ghIsIndex (Const Value :Integer; Const Str :WideString)
    :Boolean; Overload;
  Begin
    Result := (Value > 0) And (Value <= Length (Str));
  End;

  Function ghIsInt (Const Value :Extended; Const Epsilon :Extended = 0)
    :Boolean; Overload;
  Begin
    Result := ghFracZero (Value, Epsilon) = 0;
  End;

  Function ghIsInt (Const Value :String) :Boolean; Overload;
  Begin
    Result := TryStrToInt (Value, GHDUInt);
  End;

  Function ghIsLower (Const Value :String) :Boolean;
  Begin
    Result := Value = ANSILowerCase (Value);
  End;

  Function ghISODate (Const Value :TDateTime) :String; Overload;
  Begin
    Result := FormatDateTime (ghfmISODate, Value);
  End;

  Function ghISODate :String; Overload;
  Begin
    Result := ghISODate (Date);
  End;

  Function ghISODateTime (Const Value :TDateTime) :String; Overload;
  Begin
    Result := FormatDateTime (ghfmISODateTime, Value);
  End;

  Function ghISODateTime :String; Overload;
  Begin
    Result := ghISODateTime (Now);
  End;

  Function ghISOTime (Const Value :TDateTime) :String; Overload;
  Begin
    Result := FormatDateTime (ghfmISOTime, Value);
  End;

  Function ghISOTime :String; Overload;
  Begin
    Result := ghISOTime (Time);
  End;

  Function ghIsPtr (Const Value :String) :Boolean;
  Begin
    Result := (Length (Value) = 8) And ghIsHex (Value);
  End;

  Function ghIsUpper (Const Value :String) :Boolean;
  Begin
    Result := Value = ANSIUpperCase (Value);
  End;

  Function ghIsZeroOr (Const Value, Other :Extended;
    Const Epsilon :Extended = 0) :Boolean;
  Begin
    { True if Value equals to 0, 0 +/- Epsilon, Other or Other +/- Epsilon.
      NOTE: Epsilon 0 -> Math.ExtendedResolution. }
    Result := IsZero (Value, Epsilon) Or SameValue (Value, Other, Epsilon);
  End;

  Function ghLastErrorMsg :String;
  Begin
    Result := SysErrorMessage (GetLastError);
  End;

  Function ghLeft (Const Value :String; Const Chr :Char;
    Const StartIndex :Integer = 1) :String; Overload;
  Begin
    Result := ghCopy (Value, StartIndex,
      ghIndexLeft (Value, Chr, StartIndex));
  End;

  Function ghLeft (Const Value, SubStr :String;
    Const StartIndex :Integer = 1;
    Const CompareType :TghStrCompare = ghscLocaleAnyCase) :String;
    Overload;
  Begin
    Result := ghCopy (Value, StartIndex,
      ghPosLeft (Value, SubStr, StartIndex, CompareType));
  End;

  Function ghLeft (Const Value, SubStr :String;
    Const CompareType :TghStrCompare) :String; Overload;
  Begin
    Result := ghLeft (Value, SubStr, 1, CompareType);
  End;

  Function ghLength (Const Value :Variant; Const Dim :Integer = 1)
    :Integer;
  Begin
    If VarIsArray (Value) Then
      Result := ghLengthDim (Value, Dim)
    Else
      Result := 1;
  End;

  Function ghLengthAll (Const Value :Variant) :Integer;
  Var
    I :Integer;
  Begin
    Result := VarArrayDimCount (Value);

    If Result = 0 Then
      Exit;

    I := Result;
    Result := 1;

    For I := 1 To I Do
      Result := Result * ghLengthDim (Value, I)
  End;

  Function ghLengthDim (Const Value :Variant; Const Dim :Integer) :Integer;
  Begin
    Result := Succ (VarArrayHighBound (Value, Dim) -
      VarArrayLowBound (Value, Dim));
  End;

  Function ghLow (Const Value :Variant; Const Dim :Integer = 1) :Integer;
  Begin
    If VarIsArray (Value) Then
      Result := VarArrayLowBound (Value, Dim)
    Else
      Result := 0;
  End;

  Function ghMax (Const Value1, Value2 :TghTwoInts) :TghTwoInts;
  Begin
    Result := TghTwoInts (Max (Value1.Int64Value, Value2.Int64Value));
  End;

  Function ghMaxSplitInt (Const Value :String;
    Const Separator :String = ' '; Const IncludeEmpties :Boolean = False)
    :Integer;
  Begin
    If Value <> '' Then
      Result := MaxIntValue (
        ghSplitInts (Value, Separator, IncludeEmpties))
    Else
      Result := 0;
  End;

  Function ghMsgPending (Const Window :THandle;
    Const MinID, MaxID :Integer) :Boolean; Overload;
  Var
    Msg :TMsg;
  Begin
    Result := ghPeekMsg (Msg, Window, MinID, MaxID);
  End;

  Function ghMsgPending (Const Window :THandle = 0; Const ID :Integer = 0)
    :Boolean; Overload;
  Begin
    Result := ghMsgPending (Window, ID, ID);
  End;

  Function ghMethodRec (Const Obj, Address :Pointer) :TMethod;
  Begin
    Result.Data := Obj;
    Result.Code := Address;
  End;

  Function ghMin (Const Value1, Value2 :TghTwoInts) :TghTwoInts;
  Begin
    Result := TghTwoInts (Min (Value1.Int64Value, Value2.Int64Value));
  End;

  Function ghMinMax (Const Value1, Value2 :Integer) :TghTwoInts;
  Begin
    If Value1 < Value2 Then
    Begin
      Result.Value1 := Value1;
      Result.Value2 := Value2;
    End
    Else
    Begin
      Result.Value1 := Value2;
      Result.Value2 := Value1;
    End
  End;

  Function ghMinZero (Const Value :Integer) :Integer;
  Begin
    If Value > 0 Then
      Result := Value
    Else
      Result := 0;
  End;

  Function ghModulePath (Const Module :THandle) :String; Overload;
  Var
    Buffer :Array [0..Max_Path] Of Char;
  Begin
    SetString (Result, Buffer,
      GetModuleFileName (Module, Buffer, Length (Buffer)));
  End;

  Function ghModulePath :String; Overload;
  Begin
    Result := ghModulePath (HInstance);
  End;

  Function ghMoreNull (Const Value :Variant) :Boolean;
  Begin
    // Not varEmpty or varNull
    Result := FindVarData (Value).VType > varNull;
  End;

  Function ghPeekMsg (Out Msg :TMsg; Const Window :THandle;
    Const MinID, MaxID :Integer; Const Remove :Boolean = False) :Boolean;
    Overload;
  Begin
    Result := PeekMessage (Msg, Window, MinID, MaxID, Byte (Remove));
  End;

  Function ghPeekMsg (Out Msg :TMsg; Const Window :THandle = 0;
    Const ID :Integer = 0; Const Remove :Boolean = False) :Boolean;
    Overload;
  Begin
    Result := ghPeekMsg (Msg, Window, ID, ID, Remove);
  End;

  Function ghPercent (Const Value, Percentage :Extended) :Extended;
    Overload;
  Begin
    Result := Value * (Percentage / 100);
  End;

  Function ghPercent (Const Value, Percentage :Extended;
    Const Decimals :Integer) :Extended; Overload;
  Begin
    Result := ghRound (ghPercent (Value, Percentage), Decimals);
  End;

  Function ghPercent100 (Const Value, Percentage :Extended) :Extended;
    Overload;
  Begin
    Result := Value * (100 / Percentage);
  End;

  Function ghPercent100 (Const Value, Percentage :Extended;
    Const Decimals :Integer) :Extended; Overload;
  Begin
    Result := ghRound (ghPercent100 (Value, Percentage), Decimals);
  End;

  Function ghPercentage (Const Value, Portion :Extended) :Extended;
    Overload;
  Begin
    If IsZero (Portion) Then
      Result := 0
    Else
      Result := (100 / Value) * Portion;
  End;

  Function ghPercentage (Const Value, Portion :Extended;
    Const Decimals :Integer) :Extended; Overload;
  Begin
    Result := ghRound (ghPercentage (Value, Portion), Decimals);
  End;

  Function ghPreparePos (Var Str1, Str2 :String;
    Const CompareType :TghStrCompare = ghscLocaleAnyCase) :Boolean;
  Begin
    If (Str1 = '') Or (Str2 = '') Then
      Result := False
    Else
    Begin
      Case CompareType Of
        ghscOrdinalAnyCase :
        Begin
          Str1 := UpperCase (Str1);
          Str2 := UpperCase (Str2);
        End;

        ghscLocaleAnyCase :
        Begin
          Str1 := ANSIUpperCase (Str1);
          Str2 := ANSIUpperCase (Str2);
        End;

        ghscSimple :
        Begin
          Str1 := ghSimpleStr (Str1);
          Str2 := ghSimpleStr (Str2);
        End;

        ghscSimpleAnyCase :
        Begin
          Str1 := ghSimpleStrUpper (Str1);
          Str2 := ghSimpleStrUpper (Str2);
        End;
      End;

      Result := True;
    End;
  End;

  Function ghPos (Value, SubStr :String; Const StartIndex :Integer = 1;
    Const CompareType :TghStrCompare = ghscLocaleAnyCase) :Integer;
    Overload;
  Begin
    If ghPreparePos (Value, SubStr, CompareType) Then
      Result := PosEx (SubStr, Value, StartIndex)
    Else
      Result := 0;
  End;

  Function ghPos (Const Value, SubStr :String;
    Const CompareType :TghStrCompare) :Integer; Overload;
  Begin
    Result := ghPos (Value, SubStr, 1, CompareType);
  End;

  Function ghPosLeft (Const Value, SubStr :String;
    Const StartIndex :Integer = 1;
    Const CompareType :TghStrCompare = ghscLocaleAnyCase) :Integer;
  Begin
    Result := ghPos (Value, SubStr, StartIndex, CompareType) - 1;

    If Result = -1 Then
      Result := Length (Value);
  End;

  Function ghPosStr (Const Values :Array Of String; Const Str :String;
    Const StartPos :Integer = 0;
    Const CompareType :TghStrCompare = ghscLocaleAnyCase) :Integer;
    Overload;
  Begin
    For Result := StartPos To High (Values) Do
      If ghEquals (Values [Result], Str, CompareType) Then
        Exit;

    Result := -1;
  End;

  Function ghPosStr (Const Values :Array Of String; Const Str :String;
    Const CompareType :TghStrCompare) :Integer; Overload;
  Begin
    Result := ghPosStr (Values, Str, 0, CompareType);
  End;

  Function ghPropObj (Const Obj :TObject; Const Name :String;
    Const Cls :TClass = Nil) :TObject; Overload;
  Begin
    If IsPublishedProp (Obj, Name) Then
      Result := GetObjectProp (Obj, Name, Cls)
    Else
      Result := Nil;
  End;

  Function ghPropObj (Const Obj :TObject; Const Cls :TClass = Nil)
    :TObject; Overload;

    // Call back routine
    Procedure GetProp (Prop :PPropInfo; Const Params :Variant;
      Var Continue :Boolean);
    Begin
      PObject (ghPtr (Params [1]))^ := GetObjectProp (
        TObject (ghPtr (Params [0])), Prop);
      Continue := False;
    End;
  Begin
    Result := Nil;
    ghEnumPropsObj (Obj, Cls, @GetProp, VarArrayOf ([ghVar (Obj), ghVar (
      @Result)]));
  End;

  Function ghProps (Const Cls :TClass) :TghPPropInfoArr; Overload;
  Begin
    SetLength (Result, GetTypeData (Cls.ClassInfo).PropCount);

    If Length (Result) > 0 Then
      GetPropInfos (Cls.ClassInfo, Pointer (Result));
  End;

  Function ghProps (Const Obj :TObject) :TghPPropInfoArr; Overload;
  Begin
    Result := ghProps (Obj.ClassType);
  End;

  Function ghPropStr (Const Obj :TObject; Const Name :String) :String;
  Begin
    Result := VarToStr (ghPropValue (Obj, Name));
  End;

  Function ghPropValue (Const Obj :TObject; Const Name :String) :Variant;
  Begin
    If IsPublishedProp (Obj, Name) Then
      Result := GetPropValue (Obj, Name, False)
    Else
      Result := Null;
  End;

  Function ghPtr (Const Value :String) :Pointer; Overload;
  Begin
    If ghIsPtr (Value) Then
      Result := Pointer (StrToInt ('$' + Value))
    Else
      Result := Nil;
  End;

  Function ghPtr (Const Value :Variant) :Pointer; Overload;
  Begin
    If ghMoreNull (Value) Then
      If VarIsStr (Value) Then
        Result := ghPtr (String (Value))
      Else
        Result := Pointer (Cardinal (Value))
    Else
      Result := Nil;
  End;

  Function ghQuotedISODate (Const Value :TDateTime) :String; Overload;
  Begin
    Result := QuotedStr (ghISODate (Value));
  End;

  Function ghQuotedISODate :String; Overload;
  Begin
    Result := ghQuotedISODate (Date);
  End;

  Function ghQuotedISODateTime (Const Value :TDateTime) :String; Overload;
  Begin
    Result := QuotedStr (ghISODateTime (Value));
  End;

  Function ghQuotedISODateTime :String; Overload;
  Begin
    Result := ghQuotedISODateTime (Now);
  End;

  Function ghQuotedISOTime (Const Value :TDateTime) :String; Overload;
  Begin
    Result := QuotedStr (ghISOTime (Value));
  End;

  Function ghQuotedISOTime :String; Overload;
  Begin
    Result := ghQuotedISOTime (Time);
  End;

  Function ghQuotedSQLDate (Const Value :TDateTime) :String; Overload;
  Begin
    Result := QuotedStr (ghSQLDate (Value));
  End;

  Function ghQuotedSQLDate :String; Overload;
  Begin
    Result := ghQuotedSQLDate (Date);
  End;

  Function ghQuotedSQLDateTime (Const Value :TDateTime) :String; Overload;
  Begin
    Result := QuotedStr (ghSQLDateTime (Value));
  End;

  Function ghQuotedSQLDateTime :String; Overload;
  Begin
    Result := ghQuotedSQLDateTime (Now);
  End;

  Function ghQuotedSQLTime (Const Value :TDateTime) :String; Overload;
  Begin
    Result := QuotedStr (ghSQLTime (Value));
  End;

  Function ghQuotedSQLTime :String; Overload;
  Begin
    Result := ghQuotedSQLTime (Time);
  End;

  {$StackFrames On}
  Procedure ghRaise (Const Error :String; Const Callers :Integer = 1);
    Overload;
  Begin
    Raise Exception.Create (Error) At ghReturnAddress (Callers);
  End;
  {$StackFrames Off}

  Procedure ghRaise (Const Error :String; Const Params :Array Of Const;
    Const Callers :Integer = 1); Overload;
  Begin
    ghRaise (Format (Error, Params), Callers + 1);
  End;

  Procedure ghRaiseIndex (Const Value :Integer;
    Const Callers :Integer = 1);
  Begin
    ghRaise (SListIndexError, [Value], Callers + 1);
  End;

  Procedure ghRaiseLastError (
    Const AFormat :String = '%s (GetLastError %d).';
    Const Callers :Integer = 1);
  Var
    Code :Integer;
  Begin
    { NOTE: We must call the GetLastError method before doing anything
      else. }
    Code := GetLastError;
    ghRaise (AFormat, [ghLastErrorMsg, Code], Callers + 1);
  End;

  Procedure ghRereadShortStr (Const Reader :TReader;
    Var ARightStr :String);
  Var
    Buffer :Array [0..255] Of Byte;
    Count, I :Integer;
  Begin
    { ghRereadShortStr allows the application to get the short string that
      has just been read with a TReader object.  NOTE: The application must
      call this function again with the returned string if it is considered
      incomplete. }

    Count := Reader.Position;
    Reader.Position := ghMinZero (Reader.Position - SizeOf (Buffer));
    Count := Count - Reader.Position;
    Reader.Read (Buffer, Count);

    // We look for the tentative length byte from right to left
    For I := Count - (Length (ARightStr) + 2) DownTo 0 Do
      If Buffer [I] = Count - Succ (I) Then
      Begin
        ARightStr := PShortString (@Buffer [I])^;
        Exit;
      End;

    ghRaise ('Short string not found in reader.');
  End;

  { NOTE: This function takes into acount only the Callers most direct
    caller routines that have an EBP stack frame.  The compiler does not
    generate stack frame for this function. }
  Function ghReturnAddress (Callers :Integer = 1) :Pointer;
  Asm
    Mov ECX, EBP  // Stack frame of the most direct caller

    @Loop:            // Repeat
      Dec EAX         // Dec (Callers)
      JZ  @Exit       // If Callers = 0 Then Break
      Mov ECX, [ECX]  // Stack frame (EBP) of previous caller
      Jmp @Loop       // Until False

    @Exit:
      Mov EAX, [ECX + 4]  // Result := Return address of "that" caller
  End;

  Function ghRightChrs (Const Value :String; Const Chrs :TSysCharSet;
    Const MaxLength :Integer = MaxInt) :String;
  Var
    I :Integer;
  Begin
    Result := '';

    For I := Length (Value) DownTo 1 Do
      If (Length (Result) < MaxLength) And (Value [I] In Chrs) Then
        Result := Value [I] + Result
      Else
        Exit;
  End;

  Function ghRightDigits (Const Value :String;
    Const MaxLength :Integer = MaxInt) :String;
  Begin
    Result := ghRightChrs (Value, ghdgDecimal, MaxLength);
  End;

  Function ghRightInt (Const Value :String;
    Const MaxLength :Integer = MaxInt) :Integer;
  Begin
    Result := ghInt (ghRightDigits (Value, MaxLength));
  End;

  Function ghRound (Const Value :Extended; Const Decimals :Integer = 0;
    Const Epsilon :Extended = 0) :Extended;
  Begin
    If (Decimals = MaxInt) Or (Value = 0) Then
      Result := Value
    Else
      If Decimals = 0 Then
        Result := Int (Value) + ghFracRound (Value, Epsilon)
      Else
        Result := ghRoundMultiple (Value, IntPower (10, -Decimals),
          Epsilon);
  End;

  Function ghRoundInt (Const Value :Extended; Const Epsilon :Extended = 0)
    :Int64;
  Begin
    Result := Trunc (ghRound (Value, 0, Epsilon));
  End;

  Function ghRoundMultiple (Const Value, Divisor :Extended;
    Const Epsilon :Extended = 0) :Extended;
  Begin
    Result := Value / Divisor;  // First we get the quotient

    // If Value is already multiple of Divisor
    If ghIsInt (Result, Epsilon) Then
      Result := Value
    Else
      // We round the quotient to integer and multiply it by Divisor
      Result := ghRound (Result, 0, Epsilon) * Divisor;
  End;

  Function ghSetChr (Var Str :String; Const Index :Integer;
    Const Value :Char) :Boolean;
  Begin
    { NOTE: A direct character assignment in a string always calls to
      UniqueString, ghSetChr first checks if specified character matches
      the character in the given position (but it does not check the
      validity of Index). }

    If Value <> Str [Index] Then
    Begin
      Str [Index] := Value;
      Result := True;
    End
    Else
      Result := False;
  End;

  Procedure ghSetSubcomponent (Const Component :TComponent;
    Const Name :String);
  Begin
    Component.SetSubComponent (True);
    Component.Name := Name;
  End;

  Function ghSimple (Const Value :Char) :Char;
  Begin
    Result := ghComposition (Value) [1];
  End;

  Function ghSimpleStr (Const Value :String;
    Const DiacriticIndex :Integer) :String; Overload;
  Begin
    Result := Value;
    ghSimplifySafe (Result, DiacriticIndex);
  End;

  Function ghSimpleStr (Const Value :String) :String; Overload;
  Var
    I :Integer;
  Begin
    Result := Value;

    For I := 1 To Length (Result) Do
      ghSetChr (Result, I, ghSimple (Result [I]));
  End;

  Function ghSimpleStrUpper (Const Value :String) :String;
  Begin
    Result := ANSIUpperCase (ghSimpleStr (Value));
  End;

  Function ghSimplify (Var Str :String; Const Index :Integer) :Boolean;
  Begin
    Result := ghSetChr (Str, Index, ghSimple (Str [Index]));
  End;

  Function ghSimplifySafe (Var Str :String; Const Index :Integer) :Boolean;
  Begin
    Result := ghIsIndex (Index, Str) And ghSimplify (Str, Index);
  End;

  Function ghSplit (Const Value :String; Const Separator :String = ' ';
    Const IncludeEmpties :Boolean = False) :TStringDynArray;
  Var
    Count, Index :Integer;
    SubStr :String;
  Begin
    Result := Nil;

    If Value = '' Then
      Exit;

    If Separator = '' Then
    Begin
      SetLength (Result, 1);
      Result [0] := Value;
    End
    Else
    Begin
      SetLength (Result, ghCountStr (Value, Separator, ghscOrdinal) +
        1);

      If Length (Result) = 1 Then
        Result [0] := Value
      Else
      Begin
        Index := 1;
        Count := 0;

        Repeat
          SubStr := ghLeft (Value, Separator, Index, ghscOrdinal);

          If (SubStr <> '') Or IncludeEmpties Then
          Begin
            Result [Count] := SubStr;
            Inc (Count);
          End;

          Inc (Index, Length (SubStr) + Length (Separator));
        Until Index > Length (Value) + 1;

        If Count < Length (Result) Then
          SetLength (Result, Count);
      End;
    End;
  End;

  Function ghSplitInts (Const Value :String; Const Separator :String = ' ';
    Const IncludeEmpties :Boolean = False) :TIntegerDynArray;
  Var
    I :Integer;
    Strs :TStringDynArray;
  Begin
    Strs := ghSplit (Value, Separator, IncludeEmpties);
    SetLength (Result, Length (Strs));

    For I := 0 To High (Strs) Do
      Result [I] := ghInt (Strs [I]);
  End;

  Function ghSplitLast (Const Value :String; Const Separator :Char = ' ';
    Const IncludeEmpties :Boolean = False) :String;
  Var
    I, EndIndex :Integer;
  Begin
    EndIndex := Length (Value);

    For I := EndIndex DownTo 1 Do
      If Value [I] = Separator Then
        If I < EndIndex Then
        Begin
          Result := ghCopy (Value, I + 1, EndIndex);
          Exit;
        End
        Else  // I = EndIndex
          If IncludeEmpties Then
          Begin
            Result := '';
            Exit;
          End
          Else
            Dec (EndIndex);

    Result := LeftStr (Value, EndIndex);
  End;

  Function ghSQLDate (Const Value :TDateTime) :String; Overload;
  Begin
    Result := FormatDateTime (GHFMSQLDate, Value);
  End;

  Function ghSQLDate :String; Overload;
  Begin
    Result := ghSQLDate (Date);
  End;

  Function ghSQLDateTime (Const Value :TDateTime) :String; Overload;
  Begin
    Result := FormatDateTime (GHFMSQLDateTime, Value);
  End;

  Function ghSQLDateTime :String; Overload;
  Begin
    Result := ghSQLDateTime (Now);
  End;

  Function ghSQLTime (Const Value :TDateTime) :String; Overload;
  Begin
    Result := FormatDateTime (GHFMSQLTime, Value);
  End;

  Function ghSQLTime :String; Overload;
  Begin
    Result := ghSQLTime (Time);
  End;

  Function ghSQLValue (Const Value :Variant) :String;
  Begin
    If ghMoreNull (Value) Then
      If VarIsStr (Value) Then
        Result := QuotedStr (Value)
      Else
        If VarIsType (Value, varDate) Then
          If Frac (Value) = 0 Then
            Result := ghQuotedSQLDate (Value)
          Else
            Result := ghQuotedSQLDateTime (Value)
        Else
          Result := Value
    Else
      Result := 'Null'
  End;

  Function ghSQLValues (Const Values :Variant) :TStringDynArray;
  Var
    I :Integer;
  Begin
    If VarIsArray (Values) Then
    Begin
      I := ghHigh (Values);
      SetLength (Result, I + 1);

      For I := 0 To I Do
        Result [I] := ghSQLValue (Values [I]);
    End
    Else
    Begin
      SetLength (Result, 1);
      Result [0] := ghSQLValue (Values);
    End;
  End;

  Function ghStdDateTime (Const Value :TDateTime) :String; Overload;
  Begin
    Result := FormatDateTime (ghfmStdDateTime, Value);
  End;

  Function ghStdDateTime :String; Overload;
  Begin
    Result := ghStdDateTime (Now);
  End;

  Function ghStr (Const Value, MinLength :Integer;
    Const PadChr :Char = '0') :String; Overload;
  Var
    AResult :PChar Absolute Result;
    I :Integer;
  Begin
    Str (Value:MinLength, Result);

    If PadChr = ' ' Then
      Exit;

    I := 0;

    While AResult [I] = ' ' Do
    Begin
      AResult [I] := PadChr;  // Direct writing on the unique string Result
      Inc (I);
    End;
  End;

  Function ghStr (Const Value :Pointer) :String; Overload;
  Begin
    Result := Format ('%p', [Value]);
  End;

  Function ghStrs (Const Values :Array Of String) :TStringDynArray;
    Overload;
  Var
    I :Integer;
  Begin
    SetLength (Result, Length (Values));

    For I := 0 To High (Values) Do
      Result [I] := Values [I];
  End;

  Function ghStrs (Const List :TStrings) :TStringDynArray; Overload;
  Var
    I :Integer;
  Begin
    SetLength (Result, List.Count);

    For I := 0 To List.Count - 1 Do
      Result [I] := List [I];
  End;

  Function ghSwapChrs (Const Value :TghTwoChrs) :TghTwoChrs;
  Begin
    Result.Value1 := Value.Value2;
    Result.Value2 := Value.Value1;
  End;

  Function ghTwoInts (Const Value1, Value2 :Integer) :TghTwoInts; Overload;
  Begin
    Result.Value1 := Value1;
    Result.Value2 := Value2;
  End;

  Function ghTwoInts (Const Values :Variant; Const StartIndex :Integer = 0)
    :TghTwoInts; Overload;
  Begin
    Result.Value1 := ghInt (Values [StartIndex]);
    Result.Value2 := ghInt (Values [StartIndex + 1]);
  End;

  Function ghType (Const Value :Char; Const Info :Integer = ct_CType1)
    :Word;
  Begin
    If Not GetStringTypeEx (SysLocale.DefaultLCID, Info, @Value, 1,
    Result) Then
      Result := 0;
  End;

  Function ghUnitDiff (Const Groups1, Units1, Groups2, Units2 :Integer;
    Const MinUnit :Integer = 0; Const MaxUnit :Integer = 9) :Integer;
    Overload;
  Begin
    Result := ghUnits (Groups1, Units1, MinUnit, MaxUnit) -
      ghUnits (Groups2, Units2, MinUnit, MaxUnit);
  End;

  Function ghUnitDiff (Const Value1, Value2 :TghTwoInts;
    Const MinUnit :Integer = 0; Const MaxUnit :Integer = 9) :Integer;
    Overload;
  Begin
    Result := ghUnits (Value1, MinUnit, MaxUnit) -
      ghUnits (Value2, MinUnit, MaxUnit);
  End;

  Function ghUnits (Const Groups, Units :Integer;
    Const MinUnit :Integer = 0; Const MaxUnit :Integer = 9) :Integer;
    Overload;
  Begin
    Result := (Groups * (MaxUnit - Pred (MinUnit))) + (Units - MinUnit);
  End;

  Function ghUnits (Const Value :TghTwoInts;
    Const MinUnit :Integer = 0; Const MaxUnit :Integer = 9) :Integer;
    Overload;
  Begin
    Result := ghUnits (Value.HighValue, Value.LowValue, MinUnit, MaxUnit);
  End;

  Function ghValid (Const Value :String; Const Chrs :TSysCharSet) :Boolean;
  Var
    I :Integer;
  Begin
    Result := True;

    For I := 1 To Length (Value) Do
      If Not (Value [I] In Chrs) Then
      Begin
        Result := False;
        Exit;
      End;
  End;

  Function ghValue (Const AVar :Variant; Const Pos :Integer) :Variant;
    Overload;
  Begin
    If VarIsArray (AVar) Then
      Result := AVar [ghLow (AVar) + Pos]
    Else
      Result := AVar;
  End;

  Function ghVar (Const Value :String) :Variant; Overload;
  Begin
    If Value = '' Then
      Result := Null
    Else
      Result := Value;
  End;

  Function ghVar (Const Value :Pointer) :Variant; Overload;
  Begin
    If Value = Nil Then
      Result := Null
    Else
      Result := Cardinal (Value);
  End;

  Function ghVar (Const Value :TVarRec) :Variant; Overload;
  Begin
    With Value Do
      Case VType Of
        vtInteger       : Result := VInteger;
        vtBoolean       : Result := VBoolean;
        vtChar          : Result := VChar;
        vtExtended      : Result := VExtended^;
        vtString        : Result := VString^;
        vtPointer       : Result := ghVar (VPointer);
        vtPChar         : Result := ANSIString (VPChar);
        vtObject        : Result := ghVar (VObject);
        vtClass         : Result := ghVar (VClass);
        vtWideChar      : Result := VWideChar;
        vtPWideChar     : Result := WideString (VPWideChar);
        vtANSIString    : Result := ANSIString (VANSIString);
        vtCurrency      : Result := VCurrency^;
        vtVariant       : Result := VVariant^;
        vtInterface     : Result := IInterface (VInterface);
        vtWideString    : Result := WideString (VWideString);
        vtInt64         : Result := VInt64^;
        vtUnicodeString : Result := String (VUnicodeString);
        Else
          Result := Null;
      End;
  End;

  Function ghVar (Const Values :Array Of Const) :Variant; Overload;
  Var
    I :Integer;
  Begin
    Result := ghVarArr (Length (Values), False);

    For I := 0 To High (Values) Do
      ghAssign (Result, I, ghVar (Values [I]));
  End;

  Function ghVarArr (Const Size :Integer;
    Const AType :TVarType = varVariant;
    Const AllowLessThanTwo :Boolean = True) :Variant; Overload;
  Begin
    If (Size > 1) Or AllowLessThanTwo Then
      Result := VarArrayCreate ([0, Size - 1], AType)
    Else
      Result := Unassigned;
  End;

  Function ghVarArr (Const Size :Integer;
    Const AllowLessThanTwo :Boolean) :Variant; Overload;
  Begin
    Result := ghVarArr (Size, varVariant, AllowLessThanTwo);
  End;

End.

