{*************************************************************************}
{ GH Freebrary                                                            }
{                                                                         }
{ GHFVCL.pas - Unit of VCL related elements.                              }
{                                                                         }
{ This software is distributed under the open source BSD license.         }
{                                                                         }
{ Copyright (c) 2002-2013, Al Gonzalez (rescatandoadelphi.blogspot.com)   }
{ All rights reserved.                                                    }
{                                                                         }
{ Redistribution and use in source and binary forms, with or without      }
{ modification, are permitted provided that the following conditions are  }
{ met:                                                                    }
{                                                                         }
{   - Redistributions of source code must retain the above copyright      }
{     notice, this list of conditions and the following disclaimer.       }
{   - Redistributions in binary form must reproduce the above copyright   }
{     notice, this list of conditions and the following disclaimer in the }
{     documentation and/or other materials provided with the              }
{     distribution.                                                       }
{   - Neither the name of Al Gonzalez nor the names of his contributors   }
{     may be used to endorse or promote products derived from this        }
{     software without specific prior written permission.                 }
{                                                                         }
{ THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS }
{ IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED   }
{ TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A         }
{ PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT      }
{ HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,  }
{ SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT        }
{ LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,   }
{ DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY   }
{ THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT     }
{ (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE   }
{ OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.    }
{*************************************************************************}

{ Please contribute to improve this library.  I am not a good programmer,
  but good programmers know what Delphi is worth.

  A.G. }

Unit GHFVCL;  { VCL }

Interface

  Uses
    Data.DB, VCL.Controls, System.Classes;

  Type
    { Clone Data Set Event type }
    TghCloneDataSetEvent = Procedure (Sender :TObject;
      DataSet :TDataSet; Var Cls :TDataSetClass;
      Out Clone :TDataSet) Of Object;

    { ghmsCloseModal type }
    TghmsCloseModal = Packed Record
      ID :Cardinal;
      ModalResult :TModalResult;
      Unused :LongInt;
      Result :LongInt;
    End;

  { Functions and procedures }

  { Clone Data Set }
  Function ghCloneDataSet (Const DataSet :TDataSet;
    Const Cls :TDataSetClass; Const Owner :TComponent = Nil;
    ExcludedProps :String = 'Active'; Const IncludeFields :Boolean = True;
    Const ExcludedFieldProps :String = '') :TDataSet; Overload;

  { Clone Data Set }
  Function ghCloneDataSet (Const DataSet :TDataSet;
    Const Owner :TComponent = Nil; Const ExcludedProps :String = 'Active';
    Const IncludeFields :Boolean = True;
    Const ExcludedFieldProps :String = '') :TDataSet; Overload;

  { Copy Fields }
  Procedure ghCopyFields (Const Source, Dest :TDataSet;
    Const ExcludedProps :String = '');

Implementation

  Uses
    GHFRTL;

  { Functions and procedures }

  Function ghCloneDataSet (Const DataSet :TDataSet;
    Const Cls :TDataSetClass; Const Owner :TComponent = Nil;
    ExcludedProps :String = 'Active'; Const IncludeFields :Boolean = True;
    Const ExcludedFieldProps :String = '') :TDataSet; Overload;
  Var
    Params :TObject;
  Begin
    { We don't copy the IndexName property when the IndexFieldNames
      property has a value (otherwise it would be lost). }
    If ghPropStr (DataSet, 'IndexFieldNames') <> '' Then
      ExcludedProps := ghConcat (ExcludedProps, 'IndexName', ';');

    Result := TDataSet (ghClone (DataSet, Cls, Owner, ExcludedProps));

    If IncludeFields Then
      ghCopyFields (DataSet, Result, ExcludedFieldProps);

    Params := ghPropObj (DataSet, TParams);

    If Params <> Nil Then
      TParams (ghPropObj (Result, TParams)).Assign (TParams (Params));
  End;

  Function ghCloneDataSet (Const DataSet :TDataSet;
    Const Owner :TComponent = Nil; Const ExcludedProps :String = 'Active';
    Const IncludeFields :Boolean = True;
    Const ExcludedFieldProps :String = '') :TDataSet; Overload;
  Begin
    Result := ghCloneDataSet (DataSet, TDataSetClass (DataSet.ClassType),
      Owner, ExcludedProps, IncludeFields, ExcludedFieldProps);
  End;

  Procedure ghCopyFields (Const Source, Dest :TDataSet;
    Const ExcludedProps :String = '');
  Var
    I :Integer;
  Begin
    With Source Do
    Begin
      For I := 0 To Fields.Count - 1 Do
        TField (ghClone (Fields [I], Dest, ExcludedProps)).DataSet := Dest;

      For I := 0 To AggFields.Count - 1 Do
        TField (ghClone (AggFields [I], Dest, ExcludedProps)).DataSet :=
          Dest;
    End;
  End;

End.

